(* ::Package:: *)

(*
	ReactionKinetics1.m is a Mathematica 7.0 package that helps you 
	to investigate, simulate, etc. (bio)chemical reactions and models...
    Copyright (C) 2010  Janos TOTH, David Papp, Attila Laszlo Nagy

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*)


(*
http://en.wikipedia.org/wiki/GNU_General_Public _License
http://www.gnu.org/licenses/gpl-3.0.html
*)


(* ::Section:: *)
(*begin*)


BeginPackage["ReactionKinetics1`",
	{"ReactionKinetics1Stoichiometry`",
	 "ReactionKinetics1DeterministicApproach`",
     "ReactionKinetics1StochasticApproach`"
	(*,
		"ReactionKinetics1Decomposition`",
		"GraphUtilities`",
		"PlotLegends`",
		"MathSBML",
		"xCeletor"   *)
			}]


(*
If[$VersionNumer<6, 
Off[DiscreteMath`Combinatorica`Star]; 
]; 

If[$VersionNumber<6, 
Needs["Graphics`"];
Needs["Graphics`Legend`"];
Needs["LinearAlgebra`MatrixManipulation`"];
Needs["Utilities`"];, Needs["PlotLegends`"]; 
]; 

Needs["JLink`"];
If[$VersionNumber>= 6, Needs["XML`"]];  (* XML` is loaded automatically in versions 4 & 5 *)

If[$VersionNumber>5,
Needs["Combinatorica`"];
Needs["DifferentialEquations`NDSolveProblems`"];
Needs["DifferentialEquations`NDSolveUtilities`"]; 
Needs["FunctionApproximations`"];
Needs["DifferentialEquations`InterpolatingFunctionAnatomy`"];
If[$VersionNumber<6, Needs["DiscreteMath`GraphPlot`"]]];
If[$VersionNumber>5.3, Needs["GraphUtilities`"]]; 

If[$VersionNumber<6, 
On[DiscreteMath`Combinatorica`Star];
];
*)


Unprotect[
	OpenReactionKineticsPalette,
	OpenReactionKineticsNamesPalette,
	ReactionRatesNotebook
];


(* ::Section:: *)
(*help messages*)


(*Off[General::shdw];*)


$ReactionKinetics1::usage = 
"\!\(\*\nStyleBox[\"ReactionsKinetics1.m\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\)\n\
	is a package that is no doubt the nicest package ever created to help you doing fine reaction kinetics.";

$ReactionKineticsVersion::usage = "\!\(\*\nStyleBox[\"$ReactionKineticsVersion\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\)\n\
	gives the version number and the date of the program you are using.";

$ReactionKineticsPackageLoaded::usage = "\!\(\*\nStyleBox[\"$ReactionKineticsPackageLoaded\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\)\n\
	returns true if the current version of package ReactionKinetics is loaded.";

OpenReactionKineticsPalette::usage = "\!\(\*\nStyleBox[\"OpenReactionKinetics\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\)\
\!\(\*\nStyleBox[\"[\",\nFontFamily->\"Courier\"]\)\!\(\*\nStyleBox[\"]\",\nFontFamily->\"Courier\"]\)\n\
	provides a palette involving the basic symbols and notations of reaction kinetics.";

OpenReactionKineticsNamesPalette::usage = "\!\(\*\nStyleBox[\"OpenReactionKineticsNamesPalette\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\)\
\!\(\*\nStyleBox[\"[\",\nFontFamily->\"Courier\"]\)\!\(\*\nStyleBox[\"]\",\nFontFamily->\"Courier\"]\)\n\
	provides a palette involving the basic names and notions of reaction kinetics.";

ReactionRatesNotebook::usage = 
"\!\(\*\nStyleBox[\"ReactionRatesNotebook\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\)\
\!\(\*\nStyleBox[\"[\",\nFontFamily->\"Courier\"]\)\!\(\*\nStyleBox[\nSubscriptBox[\(list\), \(1\)],\nFontFamily->\"Courier\",\nFontSlant->\"Italic\",\nFontSize->10]\)\
\!\(\*\nStyleBox[\",\",\nFontFamily->\"Courier\"]\)\!\(\*\nStyleBox[\nSubscriptBox[\(list\), \(2\)],\nFontFamily->\"Courier\",\nFontSlant->\"Italic\",\nFontSize->10]\)\
\!\(\*\nStyleBox[\"]\",\nFontFamily->\"Courier\"]\)\n\
	opens a new notebook in which one can easily assign rates to the corresponding reactionsteps, where \
\!\(\*\nStyleBox[\nSubscriptBox[\(list\), \(1\)],\nFontFamily->\"Courier\",\nFontSlant->\"Italic\",\nFontSize->10]\) the list of reactions and the optional \
\!\(\*\nStyleBox[\nSubscriptBox[\(list\), \(2\)],\nFontFamily->\"Courier\",\nFontSlant->\"Italic\",\nFontSize->10]\) argumentum contains the external species";

(*opcionalis parametert lehetne ugy is megadni, hogy kulon sorban a fuggvenynev megegyszer mar az opcionalissal, es uj szoveg, lsd Mma beepitett sugo*)


(* ::Section::Closed:: *)
(*error messages*)


OpenReactionKineticsPalette::badarg = "Illegal argument of function OpenReactionKineticsPalette. \
It should not be given any arguments.";

OpenReactionKineticsNamesPalette::badarg = "Illegal argument of function OpenReactionKineticsNamesPalette. \
It should not be given any arguments.";

ReactionRatesNotebook::"badarg" = "Illegal argument of function ReactionRatesNotebook.";


(* ::Section:: *)
(*global variables*)


$ReactionKineticsVersion = "1.0 [09-06-2010]";
(*USA,http://en.wikipedia.org/wiki/Date_and_time _notation _by _country*)


(* ::Section:: *)
(*main*)


Begin["`Private`"]


(* ::Subsection::Closed:: *)
(*palettes*)


OpenReactionKineticsPalette[] :=
    NotebookPut[Notebook[{
        Cell[BoxData[GridBox[
		{
		 {ButtonBox["\[Alpha]"], ButtonBox["\[Beta]"], ButtonBox["\[Gamma]"], ButtonBox["\[Delta]"], ButtonBox["\[Theta]"], ButtonBox["\[Lambda]"]},
		 {ButtonBox["\"M\""], ButtonBox["\"E\""], ButtonBox["\"R\""], ButtonBox["(\[Placeholder])"], ButtonBox["{\[Placeholder]}"], ButtonBox["[\[Placeholder]]"]},
		 {ButtonBox["\[ShortRightArrow]"], ButtonBox["\[RightArrow]"], ButtonBox["\[LongRightArrow]"], ButtonBox["\[ShortLeftArrow]"],ButtonBox["\[LeftArrow]"], ButtonBox["\[LongLeftArrow]"]},
		 {ButtonBox["\[LeftRightArrow]"], ButtonBox["\[LongLeftRightArrow]"], ButtonBox["\[Equilibrium]"], ButtonBox["\[ReverseEquilibrium]"], ButtonBox["\[RightArrowLeftArrow]"], ButtonBox["\[LeftArrowRightArrow]"]}, 
		 {ButtonBox["\[DoubleRightArrow]"], ButtonBox["\[DoubleLongRightArrow]"], ButtonBox["\[DoubleLeftArrow]"], ButtonBox["\[DoubleLongLeftArrow]"], ButtonBox["\[DoubleLeftRightArrow]"], ButtonBox["\[DoubleLongLeftRightArrow]"]},
			(*ButtonBox["\[Placeholder]\!\(\*OverscriptBox[\"\[RightArrow]\", \"\[Placeholder]\"]\)\[Placeholder]"]},*)
		 {ButtonBox[SubscriptBox["\[Placeholder]", "\[Placeholder]"]], ButtonBox[SuperscriptBox["\[Placeholder]", "\[Placeholder]"]], ButtonBox[SubsuperscriptBox["\[Placeholder]", "\[Placeholder]", "\[Placeholder]"]],
		  ButtonBox["\!\(\*SubscriptBox[\"\[InvisiblePrefixScriptBase]\", \"\[Placeholder]\"]\)\[Placeholder]"], ButtonBox["\!\(\*SuperscriptBox[\"\[InvisiblePrefixScriptBase]\", \"\[Placeholder]\"]\)\[Placeholder]"], ButtonBox["\!\(\*SubsuperscriptBox[\"\", \"\[Placeholder]\", \"\[Placeholder]\"]\)\[Placeholder]"]}
		}, 
		RowSpacings -> 1.5, ColumnSpacings -> 1.5]],
        FontFamily -> "TimesNewRoman", FontWeight -> "Normal", FontSize -> 18]}, (*Bold*)
		Enabled->True,
        CellMargins -> {{0, 0}, {0, 0}},
		WindowMargins->{{Automatic,0},{Automatic,0}},
        Editable -> True, (*False*)
        ShowCellBracket -> False,
        WindowElements -> {},
        WindowFrame -> "Palette",
        WindowFrameElements -> "CloseBox",
        WindowSize -> {Fit, Fit},
        WindowToolbars -> {},
        WindowClickSelect -> False,
        WindowTitle -> "ReactionKinetics1 Palette",
		WindowFloating->True]
];

OpenReactionKineticsPalette[__] := (Message[OpenReactionKineticsPalette::"badarg"]; $Failed);

OpenReactionKineticsNamesPalette[] := 
	NotebookPut[Notebook[{
        Cell[BoxData[GridBox[
		{
			{ButtonBox["\"atommatrix\""]},
			{ButtonBox["\"atoms\""]},
			{ButtonBox["\"complexes\""]},
			{ButtonBox["\"deficiency\""]},
			{ButtonBox["\"externalspecies\""]},
			{ButtonBox["\"fhjgraph\""]},
			{ButtonBox["\"fhjconnectedcomponents\""]},
			{ButtonBox["\"fhjstronglyconnectedcomponents\""]},
			{ButtonBox["\"reactionorder\""]},
			{ButtonBox["\"reactionsteps\""]},
			{ButtonBox["\"species\""]},
			{ButtonBox["\"variables\""]},
			{ButtonBox["\"volpertgraph\""]}
		(*  {ButtonBox["Atoms"]},
			{ButtonBox["Models"]},
			{ButtonBox["Reactions"]}  *)
		},RowSpacings ->.9, ColumnSpacings ->1.]],
        FontFamily -> "TimesNewRoman", FontWeight -> "Normal", FontSize -> 18]}, (*Bold*)
		Enabled -> True,
        CellMargins -> {{0, 0}, {0, 0}},
		WindowMargins->{{Automatic,0},{110,Automatic}},
        Editable -> False,
        ShowCellBracket -> False,
        WindowElements -> {},
        WindowFrame -> "Palette",
        WindowFrameElements -> "CloseBox",
        WindowSize -> {Fit, Fit},
        WindowToolbars -> {},
        WindowClickSelect -> False,
        WindowTitle -> "ReactionKinetics1 Names",
		WindowFloating->True,WindowElements->{"VerticalScrollBar"}]]

OpenReactionKineticsNamesPalette[__] := (Message[OpenReactionKineticsNamesPalette::"badarg"]; $Failed);

(*CreatePalette*)


(* ::Subsection:: *)
(*reaction rates notebook*)


ReactionKinetics1`Private`myReactionRatesCounter = 0;

(*Options[ReactionRatesNotebook] := {ExternalSpecies-{}};*)

ReactionRatesNotebook[{reactions__}] := ReactionRatesNotebook[{reactions},{}];

ReactionRatesNotebook[{reactions__},(externals_?OptionQ (*_Rule*) | externals_?VectorQ)] :=
	(++ReactionKinetics1`Private`myReactionRatesCounter;
	NotebookPut[Notebook[
		{
		Cell[TextData[{
			"Fill in the following table in every placeholder with the appropriate reaction rate constants or functions as well.\nEvaluate the cell when finished."}], 
			"Text", FontSize -> 15, FontColor -> RGBColor[0,0, 0], FontWeight -> Bold
			],
		Cell[BoxData[RowBox[{RowBox[{"myReactionRates" <> ToString[ReactionKinetics1`Private`myReactionRatesCounter], "=",
				RowBox[{"Last", "/@",GridBox[{ToBoxes[#], "\[Placeholder]"} &/@ 
					Check[ReactionsData[{reactions},externals,"reactionsteps"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}],
				RowLines -> True, RowSpacings -> 2.5, ColumnSpacings -> 3, ColumnAlignments -> {Center, Right}]
				}]}]}]], "Input"
			]
		},
		WindowSize -> {720, 500},
		WindowTitle -> "Reaction Rates Notebook " <> ToString[ReactionKinetics1`Private`myReactionRatesCounter]
		]
	];)

ReactionRatesNotebook[{reactions__}] := ReactionRatesNotebook[{reactions},{}];

ReactionRatesNotebook[___] := (Message[ReactionRatesNotebook::"badarg"]; $Failed)


(* ::Subsection::Closed:: *)
(*welcoming words*)


If[Length[Names["FLAGS`ECHOLOAD"]]>0,
ReactionKinetics1`Private`msgflag=ToExpression["FLAGS`ECHOLOAD"],
ReactionKinetics1`Private`msgflag=True;
];

If[ReactionKinetics1`Private`msgflag,
Print[Style["\!\(\*
StyleBox[\"ReactionKinetics\",\nFontFamily->\"Courier New\"]\) Version "<>ToString[$ReactionKineticsVersion]<>" using Mathematica Version "<>
$Version<>" (Version "<>ToString[$VersionNumber]<>", Release "<>ToString[$ReleaseNumber]<>")"<>
" loaded "<>DateString[{"Day"," ","MonthName", " ", "Year", " at ", "Hour24",":", "Minute", " ", "TimeZone"}]
<>"\nGNU General Public License (GPLv3) Terms Apply. ",FontWeight-> Bold,FontColor->Black]];
];

If[ReactionKinetics1`Private`msgflag,
Print[Style["Please report ReactionKinetics issues or problems to\n",FontWeight-> Bold,FontColor->Black], 
Hyperlink["jtoth@math.bme.hu","mailto:jtoth@math.bme.hu"],Style[", "],Hyperlink["papp.david@gmail.com","mailto:papp.david@gmail.com"],
Style[", "],Hyperlink["nagyal@math.bme.hu", "mailto:nagyal@math.bme.hu"]]
]

FLAGS`ECHOLOAD = ReactionKinetics1`Private`msgflag;
Remove[ReactionKinetics1`Private`msgflag];


End[ ];


(* ::Section:: *)
(*end*)


If[ $ReactionKineticsPackageLoaded =!= True, 
	If[ Length[Notebooks["ReactionKinetics1 Palette"]] === 0,
			OpenReactionKineticsPalette[];
	];
	If[ Length[Notebooks["ReactionKinetics1 Names"]] === 0, 
			OpenReactionKineticsNamesPalette[];
	];
];

$ReactionKineticsPackageLoaded = True;


SetAttributes[
	{
		OpenReactionKineticsPalette,
		OpenReactionKineticsNamesPalette,
		ReactionRatesNotebook
	},
	{Protected,ReadProtected}(*Locked*)
];


EndPackage[ ];


(*
Arrows:
{\[LeftArrow],\[RightArrow],\[LeftRightArrow],\[DoubleLeftRightArrow],\[ShortLeftArrow],\[ShortRightArrow],\[LongLeftRightArrow],\[DoubleLongLeftRightArrow],\[DoubleLeftArrow],\[DoubleRightArrow],\[LongLeftArrow],\[DoubleLongLeftArrow],\[LeftArrowRightArrow],\[RightArrowLeftArrow],\[LongRightArrow],\[DoubleLongRightArrow],\[ReverseEquilibrium],\[Equilibrium],->}
{LeftArrow,RightArrow,LeftRightArrow,DoubleLeftRightArrow,ShortLeftArrow,ShortRightArrow,
LongLeftRightArrow,DoubleLongLeftRightArrow,DoubleLeftArrow,DoubleRightArrow,LongLeftArrow,
DoubleLongLeftArrow,LeftArrowRightArrow,RightArrowLeftArrow,LongRightArrow,DoubleLongRightArrow,
ReverseEquilibrium,Equilibrium,Rule}
*)
