(* ::Package:: *)

BeginPackage["ReactionKinetics1DeterministicApproach`",{"ReactionKinetics1Stoichiometry`"}]


(* ::Subsubsection::Closed:: *)
(*unprotect functions, variables*)


Unprotect[
	Conditions,
	ExternalSpecies,
	Positivity,

	RightHandSide,
	DeterministicModel,
	Concentrations,
	StationaryPoints,
	MassConservation,
	EigenvaluesJacobianMatrix
];


Begin["`Private`"]


(* ::Subsubsection:: *)
(*Own functions*)


VariablesQ[x_] := And@@((Head[#]===Symbol)&/@x);


MassActionKineticFunction[alpha_,variables_,rates_] := rates*((Times @@ Power[variables,#]) &/@ Transpose[alpha]);


(* ::Subsubsection:: *)
(*RightHandSide, DeterministicModel*)


RightHandSide::badarg = "Illegal argument of function RightHandSide.";

Options[RightHandSide] := {ExternalSpecies->{}};

RightHandSide[{reactions___},rates_?VectorQ,OptionsPattern[]] := 
	RightHandSide[Sequence@@Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"\[Alpha]","\[Beta]","variables"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}],rates];

RightHandSide[alpha_?MatrixQ,beta_?MatrixQ,variables_?VectorQ,rates_?VectorQ] := Dot[beta-alpha,MassActionKineticFunction[alpha,variables,rates]];

RightHandSide[___] := (Message[RightHandSide::"badarg"]; $Failed)


DeterministicModel::badarg = "Illegal argument of function DeterministicModel.";
DeterministicModel::rates = "The size of rate constants unequal to the number of given reactionsteps.";
DeterministicModel::vars = "The type of vector of variables must equal to the number of internal species.";
DeterministicModel::func = "The given rate function does not have an appropriate domain or image dimension.";

Options[DeterministicModel] := {ExternalSpecies->{}};

DeterministicModel[{reactions__},s_Symbol:Global`t,OptionsPattern[]] :=
	DeterministicModel[{reactions},Array[Subscript[Global`r,#]&,
		Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"R"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}]],s,ExternalSpecies->OptionValue[ExternalSpecies]];

DeterministicModel[{reactions__},rates_?VectorQ,s_Symbol:Global`t,OptionsPattern[]] :=
	Module[{ alpha, beta, vars, reacs = Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"R"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}]},
			If[reacs===Length[rates],
				{alpha, beta, vars} = ReactionsData[{reactions},OptionValue[ExternalSpecies],"\[Alpha]","\[Beta]","variables"];
				vars = Through[vars[s]];
				{Thread[D[vars,s] == RightHandSide[alpha,beta,vars,rates]], vars},
				Message[DeterministicModel::"rates"];
				$Failed
			]
	];

(*?VariablesQ*)
DeterministicModel[{reactions__},rates_?VectorQ,vars_?VectorQ,s_Symbol:Global`t,OptionsPattern[]] :=
	Module[{ eq, vnew, vold },
			{eq, vold} = 
				Check[DeterministicModel[{reactions},rates,s,ExternalSpecies->OptionValue[ExternalSpecies]],Abort[];,{DeterministicModel::"rates",ReactionsData::wrreac,ReactionsData::badarg}];
			If[Length[vars]===ReactionsData[{reactions},OptionValue[ExternalSpecies],"M"],
				vnew = Through[vars[s]];
				{eq /. Thread[vold->vnew] /. Thread[D[vold,s]->D[vnew,s]], vnew},
				Message[DeterministicModel::"vars"];
				$Failed
			]
	];

(*vars van de rates nincs eset kezelese?*)

DeterministicModel[{reactions__},ratefunc_Function,s_Symbol:Global`t,OptionsPattern[]] := 
	Module[ {r, alpha, beta, vars, nonlin},
			{r, alpha, beta, vars} = Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"R","\[Alpha]","\[Beta]","variables"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
			vars = Through[vars[s]];
			nonlin = Check[ratefunc @@ vars, Message[DeterministicModel::"func"]; Abort[], {Function::"fpct",Function::"slotn",General::"stop"}];
			(*Check[ratefunc @@ Most[vars], Message[DeterministicModel::"func"]; Abort[], {Function::"fpct",Function::"slotn",General::"stop"}];*)
			If[r===Length[nonlin],
				{Thread[D[vars,s] ==  Dot[beta-alpha,nonlin]], vars},
				Message[DeterministicModel::"func"];
				$Failed
			]
	];

(*?VariablesQ*)
DeterministicModel[{reactions__},ratefunc_Function,vars_?VectorQ,s_Symbol:Global`t,OptionsPattern[]] := 
	Module[ {eq, vold, vnew},
			{eq, vold} = Check[DeterministicModel[{reactions},ratefunc,s,ExternalSpecies->OptionValue[ExternalSpecies]], Abort[];,
							{ReactionsData::wrreac,ReactionsData::badarg,Function::"fpct",Function::"slotn",General::"stop",DeterministicModel::"func"}];
			If[Length[vars]===ReactionsData[{reactions},OptionValue[ExternalSpecies],"M"],
				vnew = Through[vars[s]];
				{eq /. Thread[vold->vnew] /. Thread[D[vold,s]->D[vnew,s]], vnew},
				Message[DeterministicModel::"vars"];
				$Failed
			]
	];

DeterministicModel[___] := (Message[DeterministicModel::"badarg"]; $Failed)


(* ::Subsubsection:: *)
(*Concentrations*)


Options[Concentrations] := {ExternalSpecies->{}};

Concentrations::badarg = "Illegal argument of function Concentrations.";
Concentrations::init = "The initialization vector does not have an appropiate size.";

(* Solving the ODE symbolically *)

Concentrations[{reactions__},rates_?VectorQ,init_?VectorQ,s_Symbol:Global`t,opts___?OptionQ] := Concentrations[{reactions},rates,init,0,s,opts];

Concentrations[{reactions__},rates_?VectorQ,init_?VectorQ,vars_?VariablesQ,s_Symbol:Global`t,opts___?OptionQ] := Concentrations[{reactions},rates,init,0,vars,s,opts];

Concentrations[{reactions__},rates_?VectorQ,init_?VectorQ,t0_?NumericQ,s_Symbol:Global`t,opts___?OptionQ] :=
	Module[{kineq, vars, exs},
		exs = Sequence@@FilterRules[{opts},Options[Concentrations]];
		{kineq, vars} = Check[DeterministicModel[{reactions},rates,s,exs],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg,DeterministicModel::"rates"}];
		If[Length[init]===ReactionsData[{reactions},exs,"M"],
			{
				vars,
				First[DSolve[Join[kineq, Thread[(vars /. s -> t0) == init]], vars, s, Sequence@@FilterRules[{opts},Options[DSolve]]]]
			},
			Message[Concentrations::"init"];
			$Failed
		]
	];

(*?VariablesQ*)
Concentrations[{reactions__},rates_?VectorQ,init_?VectorQ,t0_?NumericQ,vars_?VectorQ,s_Symbol:Global`t,opts___?OptionQ] :=
	Module[{kineq, newvars, exs},
		exs = Sequence@@FilterRules[{opts},Options[Concentrations]];
		{kineq, newvars} = Check[DeterministicModel[{reactions},rates,vars,s,exs],Abort[];,
									{ReactionsData::wrreac,ReactionsData::badarg,DeterministicModel::"rates",DeterministicModel::"vars"}];
		If[Length[init]===ReactionsData[{reactions},exs,"M"],
			{
				newvars,
				First[DSolve[Join[kineq, Thread[(newvars /. s -> t0) == init]], newvars, s, Sequence@@FilterRules[{opts},Options[DSolve]]]]
			},
			Message[Concentrations::"init"];
			$Failed
		]
	];

Concentrations[{reactions__},ratefunc_Function,init_?VectorQ,t0_?NumericQ,s_Symbol:Global`t,opts___?OptionQ] :=
	Module[{kineq, vars, exs},
		exs = Sequence@@FilterRules[{opts},Options[Concentrations]];
		{kineq, vars} = Check[DeterministicModel[{reactions},ratefunc,s,exs],Abort[];,
									{ReactionsData::wrreac,ReactionsData::badarg,Function::"fpct",Function::"slotn",General::"stop",DeterministicModel::"func"}];
		If[Length[init]===ReactionsData[{reactions},exs,"M"],
			vars = Through[vars[s]];
			{
				vars,
				First[DSolve[Join[kineq, Thread[(vars /. s -> t0) == init]], vars, s, Sequence@@FilterRules[{opts},Options[DSolve]]]]
			},
			Message[Concentrations::"init"];
			$Failed
		]
	];

(*?VariablesQ*)
Concentrations[{reactions__},ratefunc_Function,init_?VectorQ,t0_?NumericQ,vars_?VectorQ,s_Symbol:Global`t,opts___?OptionQ] :=
	Module[{kineq, newvars, exs},
		exs = Sequence@@FilterRules[{opts},Options[Concentrations]];
		{kineq, newvars} = Check[DeterministicModel[{reactions},ratefunc,vars,s,exs],Abort[];,
									{ReactionsData::wrreac,ReactionsData::badarg,Function::"fpct",Function::"slotn",General::"stop",DeterministicModel::"func"}];
		If[Length[init]===ReactionsData[{reactions},exs,"M"],
			{
				newvars,
				First[DSolve[Join[kineq, Thread[(newvars /. s -> t0) == init]], newvars, s, Sequence@@FilterRules[{opts},Options[DSolve]]]]
			},
			Message[Concentrations::"init"];
			$Failed
		]
	];

(* Solving the ODE numerically *)

Concentrations[{reactions__},rates_?VectorQ,init_?VectorQ,{t0_?NumericQ,t1_?NumericQ},s_Symbol:Global`t,opts___?OptionQ] :=
	Module[{kineq, vars, exs},
			exs = Sequence@@FilterRules[{opts},Options[Concentrations]];
			{kineq, vars} = Check[DeterministicModel[{reactions},rates,s,exs],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg,DeterministicModel::"rates"}];
			If[Length[init]===ReactionsData[{reactions},exs,"M"],
				{ 
					vars,
					First[(*vars/.*)NDSolve[Join[kineq,Thread[(vars /. s -> t0) == init]], vars, {s, t0, t1}, Sequence@@FilterRules[{opts},Options[NDSolve]]]]
				},
				Message[Concentrations::"init"];
				$Failed
			]
	];

Concentrations[{reactions__},rates_?VectorQ,init_?VectorQ,{t0_?NumericQ,t1_?NumericQ},vars_?VariablesQ,s_Symbol:Global`t,opts___?OptionQ] :=
	Module[{kineq, newvars, exs},
			exs = Sequence@@FilterRules[{opts},Options[Concentrations]];
			{kineq, newvars} = Check[DeterministicModel[{reactions},rates,vars,s,exs],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg,DeterministicModel::"rates",DeterministicModel::"vars"}];
			If[Length[init]===ReactionsData[{reactions},exs,"M"],
				{ 
					newvars,
					First[NDSolve[Join[kineq,Thread[(newvars /. s -> t0) == init]], newvars, {s, t0, t1}, Sequence@@FilterRules[{opts},Options[NDSolve]]]]
				},
				Message[Concentrations::"init"];
				$Failed
			]
	];

(*vars-os valtozat, lent is, fent is*)

Concentrations[___] := (Message[Concentrations::"badarg"]; $Failed)


(* ::Subsubsection:: *)
(*StationaryPoints, MassConservation, EigenvaluesJacobianMatrix*)


MyToRules[list_] := Flatten[If[Head[#]===Equal,ToRules[#],#]&/@list];


Options[StationaryPoints] := {ExternalSpecies->{}, Positivity->False, Conditions->{}};

StationaryPoints::badarg = "Illegal argument of function StationaryPoints.";
StationaryPoints::arglth = "One of the vector arguments `1`, `2` or `3` seem to have wrong sizes";
StationaryPoints::noeql = "The current reaction simplex has no stationary points inside";

StationaryPoints[{reactions__},opts___?OptionQ] := 
	Module[{r, vars},
		{r, vars} = Check[ReactionsData[{reactions},Sequence@@FilterRules[{opts},ExternalSpecies],"R","variables"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
		StationaryPoints[{reactions},Array[Subscript[Global`r,#]&,r],Superscript[#,0]&/@vars,vars,opts]
	];

StationaryPoints[{reactions__},rates_?VectorQ,opts___?OptionQ] := 
	Module[{vars},
		vars = Check[ReactionsData[{reactions},Sequence@@FilterRules[{opts},ExternalSpecies],"variables"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
		StationaryPoints[{reactions},rates,Superscript[#,0]&/@vars,vars,opts]
	];

StationaryPoints[{reactions__},rates_?VectorQ,init_?VectorQ,opts___?OptionQ] := 
	StationaryPoints[{reactions},rates,init,Check[ReactionsData[{reactions},Sequence@@FilterRules[{opts},ExternalSpecies],"variables"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}],opts];

StationaryPoints[{reactions__},rates_?VectorQ,init_?VectorQ,vars_?VectorQ,opts___?OptionQ]:=
	Module[{ equilibriums, ropts, alpha, beta, m, r, v, y, z, statp, balances, rateconst, inimass, nonneg, conds, sol, fless },
			ropts = Sequence@@FilterRules[{opts},Options[Reduce]];
			{alpha, beta, m, r} = 
					Check[ReactionsData[{reactions},Sequence@@FilterRules[{opts},ExternalSpecies],"\[Alpha]","\[Beta]","M","R"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
			fless = If[(Positivity/.{opts})/.Positivity->False,Less,LessEqual];
			If[r===Length[rates]&&m===Length[init]&&m===Length[vars],
				v=Superscript[#,"*"]& /@ vars;
				z=Subscript[y,ToString[#]]&/@Range[r];
				{statp,balances,rateconst,inimass,nonneg} = 
					Thread/@{RightHandSide[alpha,beta,v,rates] == 0,(beta-alpha).z==v-init,rates > 0,Cases[init,Except[0]]>0,fless[0,v]};
				conds=Join[balances,rateconst,inimass,nonneg];
				If[(sol=Reduce[Join[statp,conds,(Conditions/.{opts})/.Conditions->{}],Join[v,z],Reals,Backsubstitution->True,ropts])===False,
					Message[StationaryPoints::noeql];
					$Failed,
					Join[{v},{Sort[Sort[MyToRules[Select[#/.And->List,Not[MemberQ[z,First[#]]]&]]]&/@Flatten[{sol/.Thread[rateconst->True]/.Thread[inimass->True]/.Or->List}]]}]
				],
				Message[StationaryPoints::arglth,rates,init,vars];
				$Failed
			]
	];

(*vars-os StatPoints ???*)

StationaryPoints[___] := (Message[StationaryPoints::"badarg"]; $Failed)


Options[MassConservation] := {ExternalSpecies->{}};

MassConservation::badarg = "Illegal argument of function MassConservation.";
MassConservation::arglth = "One of the vector arguments `1` or `2` seem to have wrong sizes. Both should have the size of internal species.";
MassConservation::nomass = "There is no mass to be conserved.";

MassConservation[{reactions__},init_?VectorQ,opts___?OptionQ] := 
	MassConservation[{reactions},init,Check[ReactionsData[{reactions},Sequence@@FilterRules[{opts},ExternalSpecies],"variables"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}],opts]

MassConservation[{reactions__},init_?VectorQ,vars_?VectorQ,opts___?OptionQ] :=
	Module[ {alpha, beta, m, ns, ropts, inimass, sol},
			{alpha, beta, m} = Check[ReactionsData[{reactions},Sequence@@FilterRules[{opts},ExternalSpecies],"\[Alpha]","\[Beta]","M"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
			If[m===Length[init]&&m===Length[vars],
				ns = NullSpace[Transpose[beta-alpha]];
				ropts = Sequence@@FilterRules[{opts},Options[Reduce]];
				(*{mass, inimass, nonneg} = Thread /@ {ns.(vars - init) == 0, Cases[init,Except[0]]>0, vars >= 0 };*)
				inimass = Cases[init,Except[0]]>0;
				If[ns === {} || (sol=Reduce[Join@@(Thread /@ {ns.(vars - init) == 0, inimass, vars >= 0 }),vars,Reals,Backsubstitution->True,ropts])===False, 
					Message[MassConservation::nomass];
					$Failed,
					Join[{vars},{Sort[Sort[MyToRules[#/.And->List]]&/@Flatten[{sol/.Thread[inimass->True]/.Or->List}]]}]
				],
				Message[MassConservation::"arglth",init,vars];
				$Failed
			]
	];

MassConservation[___] := (Message[MassConservation::"badarg"]; $Failed)


EigenvaluesJacobianMatrix::badarg = "Illegal argument of function EigenvaluesJacobianMatrix.";
EigenvaluesJacobianMatrix::arglth = "One of the arguments `1` or `2` seem to have wrong sizes. First one should have the size of reaction steps \
while the latter one of internal species.";

Options[EigenvaluesJacobianMatrix] := {ExternalSpecies->{}};

EigenvaluesJacobianMatrix[{reactions__},OptionsPattern[]] := 
	Module[{r, vars},
		{r, vars} = Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"R","variables"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
		EigenvaluesJacobianMatrix[{reactions},Array[Subscript[Global`r,#]&,r],vars,ExternalSpecies->OptionValue[ExternalSpecies]]
	];

EigenvaluesJacobianMatrix[{reactions___},rates_?VectorQ,OptionsPattern[]] := 
	EigenvaluesJacobianMatrix[{reactions},rates,
			Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"variables"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}],
				ExternalSpecies->OptionValue[ExternalSpecies]];

EigenvaluesJacobianMatrix[{reactions___},rates_?VectorQ,vars_?VectorQ,OptionsPattern[]] := 
	Module[{alpha, beta, m, r, newvars},
		{alpha, beta, m, r} = Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"\[Alpha]","\[Beta]","M","R"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
		If[r===Length[rates]&&m===Length[vars],
			newvars = Superscript[#,"*"]& /@ vars;
			Eigenvalues[D[RightHandSide[alpha,beta,newvars,rates],{newvars}]],
			Message[EigenvaluesJacobianMatrix::arglth,rates,vars];
			$Failed
		]
	];

EigenvaluesJacobianMatrix[___] := (Message[EigenvaluesJacobianMatrix::"badarg"]; $Failed)


End[ ];


(* ::Subsubsection::Closed:: *)
(*protect functions*)


Protect[
		Conditions,
		ExternalSpecies,
		Positivity
];


SetAttributes[
	{
		RightHandSide,
		DeterministicModel,
		Concentrations,
		StationaryPoints,
		MassConservation,
		EigenvaluesJacobianMatrix
	},
	{Protected,ReadProtected}
];


(* ::Subsubsection:: *)
(*end*)


EndPackage[ ];
