(* ::Package:: *)

BeginPackage["ReactionKinetics1Problems`"]


Unprotect[
	Global`aExplodator,
	Global`bExplodator,
	Global`fOregonator
];


Unprotect[
	Models,
	Reactions,
	GetProblem,
	BioChemicalReactions
];


Models::usage=
"\!\(\*\nStyleBox[\"Models\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\) \
is a list of names of widespread mechanisms of \
reaction kinetics such as the Lotka-Volterra model, the Brusselator, the Explodator etc.";

Reactions::usage=
"\!\(\*\nStyleBox[\"Reactions\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\) \
is a list of widespread mechanisms given as lists of reversible and irreversible reaction steps \
of reaction kinetics such as the Lotka-Volterra model, the Brusselator, the Explodator etc.";

GetProblem::usage=
"\!\(\*\nStyleBox[\"GetProblem\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\)\
\!\(\*\nStyleBox[\"[]\",\nFontFamily->\"Courier\"]\) \
gives the list \!\(\*\nStyleBox[\"Models\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\).\n\
\!\(\*\nStyleBox[\"GetProblem\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\)\
\!\(\*\nStyleBox[\"[\",\nFontFamily->\"Courier\"]\)\
\!\(\*\nStyleBox[\"expr\",\nFontFamily->\"Courier\",\nFontSlant->\"Italic\",\nFontSize->10]\)\
\!\(\*\nStyleBox[\"]\",\nFontFamily->\"Courier\"]\) \
gives the mechanism with the name \!\(\*StyleBox[\"expr\",\nFontSlant->\"Italic\",\nFontSize->10]\), \
if \!\(\*\nStyleBox[\"expr\",\nFontSlant->\"Italic\",\nFontSize->10]\) \
is a string and is one of the built-in models, otherwise it fails. See for the examples the list \
\!\(\*\nStyleBox[\"Models\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\).";
(*
\!\(\*\nStyleBox[\" GetProblem \",\n FontFamily->\" Courier \",\n FontColor->RGBColor[0, 0, 1]]\)\
\!\(\*\nStyleBox[\"[\",\n FontFamily->\" Courier \"]\)\
\!\(\*\nStyleBox[\n SubscriptBox[\(expr\), \(1\)],\n FontFamily->\" Courier \",\n FontSlant->\" Italic \",\n FontSize->10]\)\
\!\(\*\nStyleBox[\",\",\n FontFamily->\" Courier \"]\)\
\!\(\*\nStyleBox[\n SubscriptBox[\(expr\), \(2\)],\n FontFamily->\" Courier \",\n FontSlant->\" Italic \",\n FontSize->10]\)\
\!\(\*\nStyleBox[\",\[Ellipsis] \",\n FontFamily->\" Courier \"]\)\
\!\(\*\nStyleBox[\"]\",\n FontFamily->\" Courier \"]\) \
gets the problems of \!\(\*\nStyleBox[\n SubscriptBox[\(expr\), \(1\)],\n FontFamily->\" Courier \",\n FontSlant->\" Italic \",\n FontSize->10]\), \
\!\(\*\nStyleBox[\n SubscriptBox[\(expr\), \(2\)],\n FontFamily->\" Courier \",\n FontSlant->\" Italic \",\n FontSize->10]\), \[Ellipsis] \n\
*)


Begin["`Private`"]


Models := 
{
	"Autocatalator",
	"Belousov-Zhabotinsky",
	"Briggs-Rauscher",
	"Brusselator",
	"Decomposition",
	"Dissociation",
	"Eigen", 
	"Explodator",
(*	"Explosion",*)
	"Feinberg-Horn I",
	"Feinberg-Horn II",
	"FKN mechanism",
	"Hudson-R\[ODoubleDot]ssler",
	"Hyver",
	"Inflow",
	"Ivanova",
	"Kliemann",
	"Leonard-Reichl",
	"Lotka",
	"Lotka-Volterra",
	"Lotka-Volterra Cyclic",
	"Michaelis-Menten",
	"Oregonator",
(*  "Oscillatory model I (92)",             
	"Oscillatory model II (137)",*)
	"Outflow",
	"Schl\[ODoubleDot]gl",
	"Triangle reaction Irreversible",
	"Triangle reaction Reversible",
	"Verhulst",
	"Volpert",
	"Willamowski-R\[ODoubleDot]ssler"
};


Reactions :=
Thread[
	Rule[Models,
		{
	(* Autocatalator *) 
{"A"->"X","X"->"Y","X"+2"Y"->3"Y","Y"->"P"}, 
(*externals: "A","P"*)

	(* Belousov-Zhabotinsky *) 
{"X"+"Y"+"H"->2"V","Y"+"A"+2"H"->"X"+"V",2"X"->"V","X"+2"A"+2"H"->2"X"+2"Z",2"X"+2"Z"->"X","Z"+"M"->"Q","Z"+"V"->"Y","V"->"Y"}, 
(*externals: "H","A","M","Q"*)
	(*{X->Tooltip[X,"Subscript[HBrO, 2]"],Y->Tooltip[Y,"Br^-"],H->Tooltip[H,"H^+"],
		A->Tooltip[A,"\!\*SubsuperscriptBox[\(BrO\), \(3\), \(-\)]"],M->Tooltip[M,"Subscript[CH, 2](COOHSubscript[), 2]"],
		V->Tooltip[V,"Ce^(4+)"]}????*)

	(* Briggs-Rauscher *) 
{"IO3 -"+2"H2O2"+"CH2(CO2H)2"+"H +"->"ICH(CO2H)2"+2"O2"+3"H2O","IO3 -"+2"H2O2"+"H +"->"HIO"+2"O2"+2"H2O","HIO"+"CH2(CO2H)2"->"ICH(CO2H)2"+"H2O",
"IO3 -"+"I -"+2"H +"->"HIO2"+"HIO","HIO2"+"I -"+"H +"->2"HIO","HIO"+"H2O2"->"I -"+"O2"+"H +"+"H2O","IO3 -"+"HIO2"+"H +"->2"IO2"+"H2O",
"IO2"+"Mn 2+"+"H2O"->"HIO2"+"Mn(OH) 2+","Mn 2+"+"H2O2"->"Mn 2+"+"H2O"+"HO2",2"HO2"->"H2O2"+"O2",2"HIO2"->"IO3 -"+"HIO"+"H +",
"I -"+"HIO"+"H +"->"I2"+"H2O","I2"+"CH2(CO2H)2"->"ICH(CO2H)2"+"H +"+"I -"},
(*http://www.chem.leeds.ac.uk/delights/texts/expt_11.html*)

	(* Brusselator *) 
{"X"\[LeftArrow]"A","B"+"X"->"Y"+"D","Y"+2"X"->3"X","X"->"E"}, 
(*externals: "A","B","D","E"*)

	(* Decomposition *)
{"C"->"A"+"B"},

	(* Dissociation *)
{"A"+"B"\[ReverseEquilibrium]"C"},

	(* Eigen *) 
{"X"->0,"X"->2"X"},
	
	(* Explodator *) 
{"A"+"X"->(1+Global`aExplodator)"X","X"+"Y"->"Z","Z"->(1+Global`bExplodator)"Y","Y"->"P"},
(*externals: "A","P", aExplodator=bExplodator=1*)

	(* Feinberg-Horn I *)
{3"X"->"X"+2"Y"->3"Y"->2"X"+"Y"->3"X"},

	(* Feinberg-Horn II *) 
{"X"->"Y","X"+"Z"->"T"->"Y"+"U"->"X"+"Z"},

		  (*{"X"\[Equilibrium]"Y","X"+"Z"->"T"->"Y"+"U"->"X"+"Z"},???????
			{"X"+"Y"\[Equilibrium]"U"+"V","X"+"U"\[Equilibrium]"Y"+"V"}       ???????*)

	(* FKN mechanism *)
{"HBrO2"+"Br -"->2"HOBr","Br -"+"BrO3 -"->"HBrO2"+"HOBr",2"HBrO2"->"BrO3 -"+"HOBr","HOBr"+"Br -"->"Br2"+"H2O","BrO3 -"+"HBrO2"->2"BrO2"+"H2O",
"HBrO2"+"H2O"->"BrO3 -","Br2"+"MA"->"BrMA"+"Br -","MA"+2"H2O"->C+2"CO2","BrMA"+2"H2O"->"Br -"+"HCO2H"+2"CO2"},

	(* Hudson-R\[ODoubleDot]ssler *) 
{"P"->"A","Q"->"B","A"+"B"->2"B","B"->"R","A"\[LeftRightArrow]"C"}, 
(*externals: "P","Q","R"*)

	(* Hyver *) 
{},

	(* Inflow *)
{"A"->"X"},
(*externals: "A"*)

	(* Ivanova *)
{"X"+"Y"->2"Y","Y"+"Z"->2"Z","X"+"Z"->2"X"},

	(* Kliemann *)
{"X"\[Equilibrium]"Y",2"X"\[Equilibrium]"X"+"Y"\[Equilibrium]2"Y"},

	(* Leonard-Reichl *) 
{2"X"\[LeftArrowRightArrow]"Y"},

	(* Lotka *) 
{"A"->"X","X"+"Y"->2"Y","Y"->"P"},

	(* Lotka-Volterra *)
{"A"+"X"->2"X","X"+"Y"->2"Y","B"\[LeftArrow]"Y"},

	(* Lotka-Volterra Cyclic *)
{"A"+"B"->2"A","B"+"C"->2"B","C"+"A"->2"C"},

	(* Michaelis-Menten *)
{"E"+"S"\[Equilibrium]"C"->"E"+"P"},

	(* Oregonator *) 
{"A"+"Y"->"X","X"+"Y"->"P","B"+"X"->2"X"+"Z",2"X"->"Q","Z"->Global`fOregonator*"Y"},

		(*  {"A"->"X","B"+"X"->2"X","X"->"P","C"+"X"->"P","Y"->"X","D"+"Y"->"X",2"X"->"Q","X"+"Y"->"P"},
			{"X"\[Equilibrium]"Y","X"+"Y"\[Equilibrium]2"Y","X"+"Y"\[Equilibrium]2"X"},?????*)

	(* Outflow *)
{"X"->"0"},
(*externals: "A"*)

	(* Schl\[ODoubleDot]gl *) 
{0\[Equilibrium]"X",2"X"\[Equilibrium]3"X"},

	(* Triangle reaction Irreversible *)
{"A"->"B"\[RightArrow]"C"->"A"},

	(* Triangle reaction Reversible *)
{"A"\[ReverseEquilibrium]"B"\[LeftRightArrow]"C"\[RightArrowLeftArrow]"A"},

	(* Verhulst *)
{"X"\[Equilibrium]2"X"},

	(* Volpert *)
{"X"+"Y"->"T","Y"+"Z"->"U"},

	(* Willamowski-R\[ODoubleDot]ssler *)
{"A"+"X"\[ReverseEquilibrium]2"X","X"+"Y"\[ReverseEquilibrium]2"Y","Q"+"Y"\[ReverseEquilibrium]"B","X"+"Z"\[ReverseEquilibrium]"C","P"+"Z"\[ReverseEquilibrium]2"Z"}
(*http://delfin.klte.hu/~gasparv/doktori/kaosz.html*)
		}
		]
];


BioChemicalReactions=
Thread[{"Calvin Cycle (Photosynthesis)", "Glycolysis", "Glyoxylate Cycle", "Szent-Gy\[ODoubleDot]rgyi\[Dash]Krebs Cycle"}->
{
{"DG3P"->"GP","DF1,6BP"->"GP"+"DG3P","DF1,6BP"+"H2O"->"DF6P"+"Pi","S7P"+"DG3P"->"DR5P"+"DX5P","DR5P"->"DRL5P","DRL5P"->"DX5P",
"ATP"+"DRL5P"->"ADP"+"DRL1,5BP","3PDG"+2"H+"->"DRL1,5BP"+"CO2"+"H2O" ,"ATP"+"3PDG"->"ADP"+"3PDGP","DG3P"+"Pi"+"NADP+"->"3PDGP"+ "NADPH" + "H+"}/.
{"DG3P"->Tooltip["DG3P","D-glyceraldehyde 3-phosphate"],
"GP"->Tooltip["GP","glycerone phosphate"],
"DRL5P"->Tooltip["DRL5P","D-ribulose 5-phosphate"],
"DR5P"->Tooltip["DR5P","D-ribose 5-phosphate"],
"DX5P"->Tooltip["DX5P","D-xylulose 5-phosphate"],
"S7P"->Tooltip["S7P","sedoheptulose 7-phosphate"],
"DF6P"->Tooltip["DF6P","D-fructose 1,6-bisphosphate"],
"DG3P"->Tooltip["DG3P","D-glyceraldehyde 3-phosphate"],
"DF1,6BP"->Tooltip["DF1,6BP","D-fructose 1,6-bisphosphate"],
"3PDG"->Tooltip["3PDG","3-phospho-D-glycerate"],
"Pi"->Tooltip["Pi","phosphate"],
"DRL1,5BP"->Tooltip["DRL1,5BP","D-ribulose 1,5-bisphosphate"],
"3PDGP"->Tooltip["3PDGP","3-phospho-D-glyceroyl phosphate"],
"H2O"->Tooltip["H2O","dihydrogen monoxide"],
"H+"->Tooltip["H+","hydrogen ion"],
"CO2"->Tooltip["CO2","carbon dioxide"],
"ATP"->Tooltip["ATP","adenosine-5'-triphosphate"],
"ADP"->Tooltip["ADP","adenosine diphosphate"],
"NADP+"->Tooltip["NADP+","nicotinamide adenine dinucleotide phosphate"],
"NADPH"->Tooltip["NADPH","NADPH"](*???*)},

{"Glc"+"ATP"-> "G6P"+"ADP"+"H+","G6P"\[Equilibrium]"F6P","F6P"+"ATP"->"ADP"+"F1,6BP","F1,6BP"\[RightArrowLeftArrow]"GADP"+"DHAP","DHAP"\[Equilibrium]"GADP",
"GADP"+"NAD+"+"Pi"-> "NADH"+"H+"+"1,3BPG","1,3BPG"->"GADP","1,3BPG"+"ADP"->"ATP"+"3PG","3PG"\[ReverseEquilibrium]"2PG",
"2PG"->"PEP"+"H2O","PEP"->"2PG","PEP"+"ADP"-> "ATP"+"Pyr"}/.
{"Glc"->Tooltip["Glc","D-glucose"],
"G6P"->Tooltip["G6P","\[Alpha]-D-glucose-6-phosphate"],
"F6P"->Tooltip["F6P","\[Beta]-D-fructose 6-phosphate"],
"Pi"->Tooltip["Pi","phosphate"],
"F1,6BP"->Tooltip["F1,6BP","\[Beta]-D-fructose 1,6-bisphosphate"],
"GADP"->Tooltip["GADP","D-glyceraldehyde 3-phosphate"],
"DHAP"->Tooltip["DHAP","dihydroxyacetone phosphate"],
"1,3BPG"->Tooltip["1,3BPG","D-1,3-bisphosphoglycerate"],
"3PG"->Tooltip["3PG","3-phosphoglycerate"],
"2PG"->Tooltip["2PG","2-phosphoglycerate"],
"PEP"->Tooltip["PEP","phosphoenolpyruvate"],
"Pyr"->Tooltip["Pyr","pyruvate"],
"H+"->Tooltip["H+","hydrogen ion"],
"H2O"->Tooltip["H2O","dihydrogen monoxide"],
"ATP"->Tooltip["ATP","adenosine-5'-triphosphate"],
"ADP"->Tooltip["ADP","adenosine diphosphate"],
"NAD+"->Tooltip["NAD+","nicotinamide adenine dinucleotide"],
"NADH"->Tooltip["NADH","NADH"](*???*)},

{"SM"+"NAD+"->"Oxc" + "NADH" + "H+","ACoA"+"H2O"+"Oxc"->"C"+"HS-CoA"+"H+","ACoA"+"H2O"+"G"->"SM"+"HS-CoA"+"H+","IsoC"->"S"+"G","C"->"cA"+"H2O","cA"+"H2O"->"IsoC"}/.
{"Oxc"-> Tooltip["Oxc","oxalocetate"],
"C"->Tooltip["C","citrate"],
"ACoA"-> Tooltip["ACoA","acetyl CoA"],
"SM"->Tooltip["SM","S-malate"],
"S"-> Tooltip["S","succinate"],
"IsoC"->Tooltip["IsoC","isocitrate"],
"cA"->Tooltip["cA","cis-aconitate"],
"G"->Tooltip["G","glyoxylate"],
"H+"->Tooltip["H+","hydrogen ion"],
"H2O"->Tooltip["H2O","dihydrogen monoxide"],
"NAD+"->Tooltip["NAD+","nicotinamide adenine dinucleotide"],
"HS-CoA"->Tooltip["HS-CoA","coenzyme A"],
"NADH"->Tooltip["NADH","NADH"](*???*)},

{"Oc"+"ACoA"+"H2O"->"C"+"CSH","C"->"cA"+"H2O"->"IsoC","IsoC"+"NAD+"-> "Os"+"NADH"+"H+","Os"-> "\[Alpha]K"+"CO2","\[Alpha]K"+"NAD+"+"CSH"->"SCoA"+"NADH"+"H+"+"CO2",
"SCoA"+"GDP"+"Pi"->"S"+"CSH"+"GTP","S"+"Q"->"F"+"QH2","F"+"H2O"->"LM","LM"+"NAD+"->"Oc"+"NADH"+"H+"}/.
{"Oc"->Tooltip["Oc","oxalocetate"],
"ACoA"->Tooltip["ACoA","acetyl CoA"],
"C"->Tooltip["C","citrate"],
"CSH"->Tooltip["CSH","CoA-SH"],
"cA"->Tooltip["cA","cis-aconitate"],
"IsoC"->Tooltip["IsoC","isocitrate"],
"Os"->Tooltip["Os","oxalosuccinate"],
"\[Alpha]K"->Tooltip["\[Alpha]K","\[Alpha]-ketoglutarate"],
"SCoA"->Tooltip["SCoA","succinyl-CoA"],
"S"->Tooltip["S","succinate"],
"Q"->Tooltip["Q","ubiquinone"],
"F"->Tooltip["F","fumarate"],
"QH2"-> Tooltip["QH2","ubiquinol"],
"LM"->Tooltip["LM","L-malate"],
"Pi"->Tooltip["Pi","phosphate"],
"H2O"->Tooltip["H2O","dihydrogen monoxide"],
"H+"->Tooltip["H+","hydrogen ion"],
"CO2"->Tooltip["CO2","carbon dioxide"],
"NAD+"->Tooltip["NAD+","nicotinamide adenine dinucleotide"],
"GDP"->Tooltip["GDP","guanosine diphosphate"],
"GTP"->Tooltip["GTP","guanosine-5'-triphosphate"],
"NADH"->Tooltip["NADH","NADH"](*???*)}

}
];


GetProblem::badarg = "Illegal argument of function GetProblem.";
GetProblem::nvmod = "Argument '`1`' is not a valid built-in model. See for the examples the list \
\!\(\*\nStyleBox[\"Models\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\).";

GetProblem[x_?StringQ] := 
	If[
		MemberQ[Models,x],
			x /. Reactions,
			Message[GetProblem::"nvmod",x];
			$Failed
	  ];

SetAttributes[GetProblem,Listable];

GetProblem[x__?StringQ] := GetProblem[{x}];

GetProblem[] := Models;

GetProblem[___] := (Message[GetProblem::"badarg"]; $Failed)


End[ ];


Protect[
	Global`aExplodator,
	Global`bExplodator,
	Global`fOregonator
];


SetAttributes[
	{
		Models,
		Reactions,
		GetProblem,
		BioChemicalReactions
	},
	{Protected,ReadProtected}(*Locked*)
];


EndPackage[ ];
