(* ::Package:: *)

BeginPackage["ReactionKinetics1StochasticApproach`",{"ReactionKinetics1Stoichiometry`","ReactionKinetics1DeterministicApproach`","Units`","PhysicalConstants`"}]


(* ::Subsubsection::Closed:: *)
(*unprotect functions*)


Unprotect[
	Global`x,(*???*)
	Global`z,(*???*)
	Global`g,(*GeneratingFunction*)
	Global`P(*ProbabilityFunction*)
	(*Global`EExpectationFunction*)
];


Unprotect[
	Bound,
	MaxIteration,
	Verbose,
	Species,
	(* Method...?, StiffNessSwitching, slow- fast variables *)
	
	ChemicalMasterEquation,
	ProbabilityGeneratingFunctionEquation,
	SolveProbabilityGeneratingFunctionEquation,
	ExpectedvalueProcess,

	DependencyGraph,
	ShowDependencyGraph,

	Simulation,
	SimulationPlot
]


Begin["`Private`"]


(* ::Subsubsection:: *)
(*Own functions*)


MyDerivative[d_, f_, x_] := Derivative[Sequence @@ d][f][Sequence @@ x];


MassActionKineticFunction[alpha_,variables_,rates_] := rates*((Times @@ Power[variables,#]) &/@ Transpose[alpha]);


Intensity[alphatr_,X_,rates_] := (*Intensity[alphatr,X,rates] =*) rates*(Times@@FactorialPower[X,#] &/@ alphatr);


MassConservationQ[{reactions__},externals_List:{},opts___?OptionQ] := 
	Module[{gamma, vars, ropts, sol},
			{gamma, vars} = Check[ReactionsData[{reactions},externals,"\[Gamma]","variables"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
			ropts = Sequence@@FilterRules[{opts},Options[Reduce]];
			If[(sol=Reduce[Flatten[Thread/@{Transpose[gamma].vars==0,vars>0}],vars,Reals,Backsubstitution->True,ropts])=!=False,
				sol,
				$Failed
			]
	];

MyFilterRules[rule_,opts___] := (First[rule]/.FilterRules[{opts},First[rule]]) /. rule;

(*First[MyFilterRules[rule,opts]]*)


(* ::Subsubsection:: *)
(*ChemicalMasterEquation*)


Options[ChemicalMasterEquation] := {ExternalSpecies->{}};

ChemicalMasterEquation::badarg = "Illegal argument of function ChemicalMasterEquation.";
ChemicalMasterEquation::rvarg = "Argument `1` or `2` have wrong sizes. \
First one should have the length of the reaction steps while the latter one of internal species including time parameter.";

ChemicalMasterEquation[{reactions__}, h_Symbol:Global`P, OptionsPattern[]] := 
	ChemicalMasterEquation[{reactions},
		Array[Subscript[Global`r,#]&,Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"R"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}]],
			h, ExternalSpecies->OptionValue[ExternalSpecies]];

ChemicalMasterEquation[{reactions__}, rates_?VectorQ, h_Symbol:Global`P, OptionsPattern[]] := 
	ChemicalMasterEquation[{reactions},rates,
		Prepend[Array[Subscript[Global`x, #]&,Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"M"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}]],Global`t],
		h, ExternalSpecies->OptionValue[ExternalSpecies]];

ChemicalMasterEquation[{reactions__}, rates_?VectorQ, vars_?VectorQ, h_Symbol:Global`P, OptionsPattern[]] := 
	Module[{alpha, gamma, m, r, s},
			{alpha, gamma, m, r} = Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"\[Alpha]","\[Gamma]","M","R"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
			If[ Length[rates] === r && Length[vars] === m+1,
				D[h@@vars,First[vars]] == 
					-(h@@vars)*Total[FunctionExpand[Intensity[Transpose[alpha],Rest[vars],rates]]] (*Expand!!!*)
					+Total[Map[h@@(vars-Prepend[#,0])*Total[FunctionExpand[Intensity[Transpose[alpha],Rest[vars]-#,rates]]]&,Transpose[gamma]]],
				Message[ChemicalMasterEquation::"rvarg",k,vars];
				$Failed
			  ]
	];

ChemicalMasterEquation[___] := (Message[ChemicalMasterEquation::"badarg"]; $Failed)


(* ::Subsubsection:: *)
(*StationaryProbabilities*)


StationaryProbabilities[0]:=0;


(* ::Subsubsection:: *)
(*ProbabilityGeneratingFunctionEquation, SolveProbabilityGeneratingFunctionEquation*)


ProbabilityGeneratingFunctionEquation::badarg = "Illegal argument of function ProbabilityGeneratingFunctionEquation.";
ProbabilityGeneratingFunctionEquation::kvarg = "Argument `1` or `2` have wrong sizes. \
First one should have the length of the reaction steps while the latter one of internal species including time parameter.";

Options[ProbabilityGeneratingFunctionEquation] := {ExternalSpecies->{}};

ProbabilityGeneratingFunctionEquation[{reactions__}, h_Symbol:Global`g, OptionsPattern[]] := 
	ProbabilityGeneratingFunctionEquation[{reactions},
		Array[Subscript[Global`r,#]&,Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"R"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}]],
			h, ExternalSpecies->OptionValue[ExternalSpecies]];

ProbabilityGeneratingFunctionEquation[{reactions__}, k_?VectorQ, h_Symbol:Global`g, OptionsPattern[]] := 
	ProbabilityGeneratingFunctionEquation[{reactions},k,
		Prepend[Array[Subscript[Global`z, #]&,Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"M"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}]],Global`t],
		h, ExternalSpecies->OptionValue[ExternalSpecies]];

ProbabilityGeneratingFunctionEquation[{reactions__}, k_?VectorQ, vars_?VectorQ, h_Symbol:Global`g, OptionsPattern[]] := 
	Module[{alpha, beta, m, r, s},
			{alpha, beta, m, r} = Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"\[Alpha]","\[Beta]","M","R"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
			If[ Length[k] === r && Length[vars] === m+1,
				MyDerivative[UnitVector[m+1,1],h,vars] == 
					Total[Times[First[Differences[MapThread[MassActionKineticFunction[#1,#2,k]&,{{alpha,beta},Rest/@{vars,vars}}]]]
								,MyDerivative[Prepend[#,0],h,vars] &/@ Transpose[alpha]]],
				Message[ProbabilityGeneratingFunctionEquation::"kvarg",k,vars]; 
				$Failed
			  ]
	];

ProbabilityGeneratingFunctionEquation[___] := (Message[ProbabilityGeneratingFunctionEquation::"badarg"]; $Failed)


SolveProbabilityGeneratingFunctionEquation::badarg = "Illegal argument of function ProbabilityGeneratingFunctionEquation.";
SolveProbabilityGeneratingFunctionEquation::initarg = "The list of internal species and the initialization vector seem to have wrong size.";

Options[SolveProbabilityGeneratingFunctionEquation] := {ExternalSpecies->{}};

(* solve the pde symbolically *)

SolveProbabilityGeneratingFunctionEquation[{reactions__},h_Symbol:Global`g,opts___?OptionQ] := 
	SolveProbabilityGeneratingFunctionEquation[{reactions},
		Array[Subscript[Global`r,#]&,Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"R"],Abort[];,{ReactionsData::"wrreac",ReactionsData::"badarg"}]],h,opts];

SolveProbabilityGeneratingFunctionEquation[{reactions__}, k_?VectorQ, h_Symbol:Global`g, opts___?OptionQ] := 
	SolveProbabilityGeneratingFunctionEquation[{reactions},k,
		Prepend[Array[Subscript[Global`z, #]&,Check[ReactionsData[{reactions},Sequence@@FilterRules[{opts},ExternalSpecies],"M"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}]],Global`t],
			h, opts];

SolveProbabilityGeneratingFunctionEquation[{reactions__}, k_?VectorQ, vars_?VectorQ, h_Symbol:Global`g, opts___?OptionQ] := 
	Module[{pgfe},
			pgfe = Check[ProbabilityGeneratingFunctionEquation[{reactions},k,vars,h,Sequence@@FilterRules[{opts},ExternalSpecies]],Abort[];,{ProbabilityGeneratingFunctionEquation::"kvarg",ProbabilityGeneratingFunctionEquation::"badarg"}];
			{h @@ vars, Flatten[DSolve[pgfe,h @@ vars,vars,Sequence@@FilterRules[{opts},Options[DSolve]]]]}
	];

(*
SolveProbabilityGeneratingFunctionEquation[{reactions__}, k_?VectorQ, init_?VectorQ, h_Symbol:Global`g, opts___?OptionQ]:=
	SolveProbabilityGeneratingFunctionEquation[{reactions}, k, init, 
		Prepend[Array[Subscript[Global`z, #]&,Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"M"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}]],Global`t],
			h, opts];

SolveProbabilityGeneratingFunctionEquation[{reactions__}, k_?VectorQ, init_?VectorQ, vars_?VectorQ, h_Symbol:Global`g, opts___?OptionQ] := 
	Module[{pgfe},
			pgfe = Check[ProbabilityGeneratingFunctionEquation[{reactions},k,vars,h,Sequence@@FilterRules[{opts},ExternalSpecies]],Abort[];,{ProbabilityGeneratingFunctionEquation::"kvarg",ProbabilityGeneratingFunctionEquation::"badarg"}];
			If[Length[init]===Length[Rest[vars]],
				{h @@ vars, Flatten[DSolve[Append[{pgfe},(h @@ vars /. {First[vars]->1})==Times@@(Rest[vars]^init)],h @@ vars,vars,Sequence@@FilterRules[{opts},Options[DSolve]]]]},
				Message[SolveProbabilityGeneratingFunctionEquation::"initarg"]; 
				$Failed
			]
	];
*)

(*function equations!!!*)

SolveProbabilityGeneratingFunctionEquation[___] := (Message[SolveProbabilityGeneratingFunctionEquation::"badarg"]; $Failed)

(* solve the pde numerically *)


(* ::Subsubsection:: *)
(*ExpectedvalueProcess*)


ExpectedvalueProcess::badarg = "Illegal argument of function ExpectedvalueProcess.";
ExpectedvalueProcess::aarg = "Argument `1` have wrong size. It should have the size of internal species.";

Options[ExpectedvalueProcess] := { ExternalSpecies->{} };

ExpectedvalueProcess[{reactions__}, a_?VectorQ, k_?VectorQ, OptionsPattern[]] :=
	ExpectedvalueProcess[{reactions},a,k,{Global`\[DoubleStruckCapitalE],Global`t},ExternalSpecies->OptionValue[ExternalSpecies]];

ExpectedvalueProcess[{reactions__}, a_?VectorQ, k_?VectorQ, h_Symbol, OptionsPattern[]] :=
	ExpectedvalueProcess[{reactions},a,k,{h,Global`t},ExternalSpecies->OptionValue[ExternalSpecies]];

ExpectedvalueProcess[{reactions__}, a_?VectorQ, k_?VectorQ, {h_Symbol, var_Symbol}, OptionsPattern[]] := 
	Module[{pgfe, vars, newvars, exp, es },
			pgfe = Check[ProbabilityGeneratingFunctionEquation[{reactions},k,ExternalSpecies->OptionValue[ExternalSpecies]],Abort[];,{ProbabilityGeneratingFunctionEquation::kvarg,ProbabilityGeneratingFunctionEquation::badarg}];
			vars = First[pgfe] /. Derivative[x__][y_][z__]:>{z};
			If[Length[a]===Length[vars]-1,
				newvars = ToString/@Check[ReactionsData[{reactions},OptionValue[ExternalSpecies],"species"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
				exp = D[pgfe,Sequence@@Thread[List[Rest[vars],a]]]/.Thread[Rest[vars]->1]
							/. Derivative[x_,y__][f_][z__]:>D[Subscript[h,ToString[FunctionExpand[Times@@FactorialPower[newvars,{y}]]]][var],{var,x}]
							/. Global`g[Global`t,Sequence@@Table[1,{Length[a]}]]->1 /. func_[Global`t]:>func[var];
				es = Union[Cases[exp,Subscript[h,_][var],3],{Integrate[First[exp],var]}];
				{Collect[exp,es],es},
				Message[ExpectedvalueProcess::"aarg",a];
				$Failed
			]
	];

ExpectedvalueProcess[___] := (Message[ExpectedvalueProcess::"badarg"]; $Failed)


(* ::Subsubsection:: *)
(*DependencyGraph*)


DependencyGraph[alphatr_?MatrixQ,gammatr_?MatrixQ] := 
	(#->Boole[Length[Union[Flatten[(Position[#,0]&)/@{Part[gammatr,First[#]],Part[alphatr,Last[#]]}]]]<Last[Dimensions[alphatr]]]&) /@ Tuples[Range[First[Dimensions[alphatr]]],2];
(* 
SparseArray, Normal!
AdjacencyMatrix 
*)

DependencyGraph[{reactions__},externals_List:{}] :=
	Module[{ m, r, rsteps, alpha, gamma, alphatr, gammatr },
			{m, r, rsteps, alpha, gamma} = Check[ReactionsData[{reactions},externals,"M","R","reactionsteps","\[Alpha]","\[Gamma]"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
			{alphatr, gammatr} = Transpose /@ {alpha,gamma};
			Flatten[Map[If[ Length[Union[Flatten[(Position[#,0]&)/@{Part[gammatr,First[#]],Part[alphatr,Last[#]]}]]] < m,
							Rule @@ Part[rsteps,#],
							{}(**)]&,Tuples[Range[r],2]](*Flatten[Array[{#1,#2}&,{R,R}],1]*)]
		  ];

ShowDependencyGraph[{reactions__},externals_List:{},opts___?OptionQ] := GraphPlot[DependencyGraph[{reactions},externals],Sequence@@FilterRules[{opts},Options[GraphPlot]]];


(* ::Subsubsection:: *)
(*Methods*)


(*t0 ???*)
DirectMethod[{reactions__},externals_,rates_,X0_,maxtime_,maxiteration_,verbose_] :=
	Module[{it, itotal, alphatr, gammatr, nwl, pill},
		{alphatr, gammatr} = Transpose /@ Check[ReactionsData[{reactions},externals,"\[Alpha]","\[Gamma]"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
		nwl := NestWhileList[
				(pill:=#;(*sorrend, lehet a vegere*)
				# + {RandomReal[ExponentialDistribution[Total[it]]],First[RandomChoice[it->gammatr,1]]})&,
				{0,X0}, (First[#]<=maxtime && Total[it = Intensity[alphatr,Last[#],rates]]>0)&, 1, maxiteration
			   ];
		If[verbose,
			Monitor[nwl,
				Column[{Row[{ProgressIndicator[First[pill]/maxtime]," " <> ToString[Ceiling[100*First[pill]/maxtime]]<>"%"}],
					"Current time: " <> ToString[First[pill],TraditionalForm] <> " units.\nCurrent state: " <> ToString[Last[pill]] <> "."}]],
			nwl
		]
	];

MyExponentialDistribution[0] := Infinity;
MyExponentialDistribution[x_] := RandomReal[ExponentialDistribution[x]];

SetAttributes[MyExponentialDistribution,Listable];

FirstReactionStepMethod[alphatr_,gammatr_,rates_,X0_,maxtime_,maxiteration_,verbose_] := 
	Module[{itd, it, pill, nwl},
		nwl := NestWhileList[
				(pill := #;(*sorrend, lehet a vegere*)
				itd = MyExponentialDistribution[it];
				# + Through[Join[List,Part[gammatr,First[Position[itd,#]]]&][Min[itd]]])&,
				{0,X0}, (First[#]<=maxtime && Total[it=Intensity[alphatr,Last[#],rates]]>0)&, 1, maxiteration
			   ];
		If[verbose,
			Monitor[nwl,
				Column[{Row[{ProgressIndicator[First[pill]/maxtime]," " <> ToString[Ceiling[100*First[pill]/maxtime]]<>"%"}],
					"Current time: " <> ToString[First[pill],TraditionalForm] <> " units.\nCurrent state: " <> ToString[Last[pill]] <> "."}]],
			nwl
		]
	];

(*$IterationLimit, $RecursionLimit*)


(* ::Subsubsection::Closed:: *)
(*NextReactionStep (under construction)*)


(*
NextReactionStepInitialize[\[Alpha]t_?MatrixQ,\[Beta]t_?MatrixQ,rates_?VectorQ,{Time_?NumericQ,X_?VectorQ}] :=
	Module[{ dg = DependencyGraph[\[Alpha]t,\[Beta]t-\[Alpha]t], tau = MapIndexed[List,MyExponentialDistribution/@Intensity[\[Alpha]t,X,rates]] },
			(*MapIndexed[List,{a,b,c,d}]*)
			NextReactionStep[\[Alpha]t,\[Beta]t,rates,{},dg]
	];

NextReactionStep[\[Alpha]t_?MatrixQ,\[Gamma]t_?MatrixQ,dg_?VectorQ,{Time_?NumericQ,queue_List,X_?VectorQ}] :=
	Module[{ m = First[queue] },
			 {Time + First[m], queue ,X + Part[\[Gamma]t,m[[2,1]]]}
	];

Sorting[v_List] := Sort[v, #1[[1]] < #2[[1]]&];

Indexing[v_List] := MapIndexed[{#1,First[#2]}&,v];

UpdateSpecifiedIntensity[alphatr_,X_,rates_,n_] := rates*(Times@@FactorialPower[X,Part[alphatr,n]]);

NextReactionStepMethod[\[Alpha]t_?MatrixQ,\[Beta]t_?MatrixQ,Rates_?VectorQ,X0_?VectorQ,Time_?NumericQ,Iteration_?NumberQ,b_?VectorQ] :=
	Module[{dg = DependencyGraph[\[Alpha]t,\[Beta]t-\[Alpha]t], it = Intensity[\[Alpha]t,X0,Rates], itold, itnew, X = X0, t = 0, Sim, dep, pq, i, n = 0, w },
			pq = Sorting[Indexing[MyExponentialDistribution /@ it]];
			itold = Indexing[it];
			Sim = {{t,X}};
			While[ t<=Time \[And] NonNegativeStepQ[\[Alpha]t,X] (*\[And] NonNegativeBoundedStepQ[\[Alpha]t,\[Beta]t,X,b]*) \[And] n < Iteration,
					(*Print[pq];*)
				  {t, i} = First[pq];(*Min*)
				  X += Part[\[Beta]t-\[Alpha]t,i];
				  Sim = Append[Sim,{t,X}];
				  dep = Cases[dg, ({i,x_} -> 1) -> x]; (*Count[dg, {i,x_} -> 1];*)(*Flatten[Position[Part[dg,i],1]]; - nem megy*)
					(*Print[dep];*)
				  itnew = itold /. (({x_,#1}->{Rates[[#1]]*Fold[Times,1,FactorialPower[X,Part[\[Alpha]t,#1]]],#1}) &/@ dep);
					(*Print[itnew];*)
					(*Thread[{x_,#1}->{Rates*Fold[Times,1,FactorialPower[X,Part[\[Alpha]t,#1]]],#1}&[a]];*)
				  pq = Sorting[(pq /. (({x_,#1}->{t + (First[itold[[#1]]]/First[itnew[[#1]]])*(x-t),#1}) &/@ dep)) /. {t,i} -> {t + MyExponentialDistribution[First[itnew[[i]]]],i}];
						(*/. {x_,i} -> {x+MyExponentialDistribution[itnew[[i]]],i}, Boole[#1 == i]*MyExponentialDistribution[itnew[[i]]]*)
				  itold = itnew;
				  n++
			];
			Sim
	];
*)


(* ::Subsubsection:: *)
(*Simulation*)


Options[Simulation] = { Bound -> Infinity(*???*), ExternalSpecies->{}, MaxIteration -> Infinity, Method -> "Direct", Verbose -> False };

Simulation::badarg = "Illegal argument of function Simulation.";
Simulation::method = "The given '`1`' method does not in the list of available methods. Try...";
Simulation::time = "Argument `1` is not positive. The maximum time parameter is expected to be a positive real number.";
Simulation::rsarg = "Argument '`1`' or '`2`' have wrong sizes. First one is expected to be the size of reaction steps while the latter one of internal species.";
Simulation::volarg = "Argument `1` is not positive. The volume is expected to be a positive real number.";

ChangeOfUnits[V_,rates_,alpha_] := rates*((AvogadroConstant*Mole)*V)^(1 - Total/@Transpose[alpha]);

Simulation[{reactions__},rates_?VectorQ,X0_?VectorQ,maxtime_?NumericQ,V_?NumericQ,opts___?OptionQ] :=
	If[ V>0,
		Simulation[{reactions},
				   ChangeOfUnits[V,rates,Check[ReactionsData[{reactions},Sequence@@FilterRules[{opts},ExternalSpecies],"\[Alpha]"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}]],
				   X0,maxtime,Sequence@@FilterRules[{opts},Options[Simulation]]],
		Message[Simulation::"volarg",V]; 
		$Failed
	];

Simulation[{reactions__}, rates_?VectorQ, X0_?VectorQ, maxtime_?NumericQ, OptionsPattern[]] := 
	Module[{bound, exs, maxit, method, verbose, alpha, beta, gamma, order, m, r, alphatr, betatr, gammatr},
		{bound, exs, maxit, method, verbose} = {OptionValue[Bound], OptionValue[ExternalSpecies], OptionValue[MaxIteration], OptionValue[Method], OptionValue[Verbose]};
		{alpha, beta, gamma, order, m, r} = Check[ReactionsData[{reactions},exs,"\[Alpha]","\[Beta]","\[Gamma]","reactionorder","M","R"], Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
		{alphatr, betatr, gammatr} = Transpose /@ {alpha, beta, gamma};
		If[r===Length[rates] && m===Length[X0],
			If[maxtime>0,
				Switch[method,
					"Direct", 
						DirectMethod[{reactions},exs,rates,X0,maxtime,maxit,verbose],
					"FirstReactionStep",
						FirstReactionStepMethod[alphatr,gammatr,rates,X0,maxtime,maxit,verbose],
					"ModifiedFirstReactionStep", 
						$Failed,
					"NextReactionStep", 
						$Failed,
					"Tau-Leaping",
						$Failed,
					_, 
						Message[Simulation::"method",method]; 
						$Failed
				],
				Message[Simulation::"time",maxtime]; 
				$Failed
			],
			Message[Simulation::"rsarg",rates,X0]; 
			$Failed
		]
	];

Simulation[___] := (Message[Simulation::"badarg"]; $Failed)


(* ::Subsubsection:: *)
(*SimulationPlot*)


Options[SimulationPlot] = { Species -> All };

SimulationPlot::badarg = "Illegal argument of function SimulationPlot.";
SimulationPlot::species = "The given species are not the internal species of reactions `1`.";

SimulationPlot[{reactions__},rates_?VectorQ,X0_?VectorQ,maxtime_?NumericQ,V_?NumericQ,opts___?OptionQ] :=
	If[ V>0,
		SimulationPlot[{reactions},
				   ChangeOfUnits[V,rates,Check[ReactionsData[{reactions},Sequence@@FilterRules[{opts},ExternalSpecies],"\[Alpha]"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}]],
				   X0,maxtime,Sequence@@FilterRules[{opts},{Options[Simulation],Options[SimulationPlot],Options[ListLinePlot]}]],
		Message[Simulation::"volarg",V]; 
		$Failed
	];

SimulationPlot[{reactions__},rates_?VectorQ,X0_?VectorQ,maxtime_?NumericQ,opts___?OptionQ] := 
	Module[{ sp, givensp, proc, w },
			sp = Check[ReactionsData[{reactions},Sequence@@FilterRules[{opts},ExternalSpecies],"species"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
			givensp = ((Species /. FilterRules[{opts},Species]) /. Species -> All) /. All->sp;
			If[VectorQ[givensp] && (w = Flatten[Position[sp,#] &/@ givensp]) =!= {},
				proc = Check[Simulation[{reactions},rates,X0,maxtime,Sequence@@FilterRules[{opts},Options[Simulation]]],Abort[];,{Simulation::rsarg,Simulation::time,Simulation::method,Simulation::badarg}];
				(*Transpose[Map[Thread[List[#]]&,proc]][[w]], maskent kiszedve az anyagokat!!!*) (* Outer[Flatten[#2][[{1,#1+1}]]&,w,proc,1] *)
				ListLinePlot[Transpose[Map[Thread[List@@#]&,proc]][[w]],Sequence@@FilterRules[{opts},Options[ListLinePlot]]], (*Show, PlotLegend*)
				Message[SimulationPlot::"species",{reactions}];
				$Failed
			]
	];
(*ListLinePlot !!!!!*)

SimulationPlot[___] := (Message[SimulationPlot::"badarg"]; $Failed)

(*
Options[SimulationPlot2D] = { Species -> {} };

SpecSubsets[l_,w_] := Part[#,w] &/@ l;

SimulationPlot2D[Reactions___,ExternalSpecs_List:{},Rates_?VectorQ,X0_?VectorQ,Time_?NumericQ,{opts___},OptionsPattern[]] := 
	Module[{proc=Last/@Simulation[Reactions,ExternalSpecs,Rates,X0,Time],spec = species/. ReactionSpecies[Reactions,ExternalSpecs],
			opval=DeleteDuplicates[OptionValue[Species]],w},
			w = Partition[Flatten[((Position[spec,#1]&)/@#1&)/@opval],2];
			Show[ListLinePlot[(SpecSubsets[proc,#1]&)/@w,PlotLegend->(Style[ToString[#1],Bold,22]&)/@opval,opts]]
		  ];

Options[SimulationPlot3D] = { Species -> {} };

SimulationPlot3D[Reactions___,ExternalSpecs_List:{},Rates_?VectorQ,X0_?VectorQ,Time_?NumericQ,{opts___},OptionsPattern[]]:=
	Module[{proc=Last/@Simulation[Reactions,ExternalSpecs,Rates,X0,Time],spec=species/. ReactionSpecies[Reactions,ExternalSpecs],
			opval=DeleteDuplicates[OptionValue[Species]],w},
			w = Partition[Flatten[((Position[spec,#1]&)/@#1&)/@opval],3];
			Show[ListPlot3D[(SpecSubsets[proc,#1]&)/@w,opts]]
		  ];
*)


(* ::Subsubsection::Closed:: *)
(*protect functions*)


End[ ];


Unprotect[
	Global`x,
	Global`z,
	Global`g,
	Global`P
];


SetAttributes[
	{
		Bound,
		MaxIteration,
		Species,
		Verbose,
	
		ChemicalMasterEquation,
		ProbabilityGeneratingFunctionEquation,
		SolveProbabilityGeneratingFunctionEquation,
		ExpectedvalueProcess,
	
		DependencyGraph,
		ShowDependencyGraph,

		Simulation,
		SimulationPlot
	},
	{Protected, ReadProtected}
];


(* ::Subsubsection:: *)
(*end*)


EndPackage[ ];
