(* ::Package:: *)

BeginPackage["ReactionKinetics1Stoichiometry`",{"ReactionKinetics1Problems`","Combinatorica`","GraphUtilities`"}]


(* ::Subsubsection::Closed:: *)
(*unprotect functions*)


Unprotect[
	Global`atommatrix,(**)
	Global`atoms,(**)

	Global`c,(*cC or Conc?*)
	Global`r(*rate*)
];


Unprotect[
	ComplexColours,
	Data,(**)
	ExternalSpecies,
	FormattedOutput,
	StronglyConnectedComponentsColours,

	ToCanonicalForm,
	ToReversible,
	PropertiesReactionsData,
	ReactionsData,
	WeaklyReversibleQ,

	ShowFHJGraph,
	ShowVolpertGraph,

	Atoms,
	ToAtomMatrix,
	AtomConservingQ,
	AtomsQ,
	FromAtomMatrix
];


Begin["`Private`"]


(* ::Subsubsection:: *)
(*ReactionsData own functions*)


(*
ReactionQ ???
NotListQ := (Head[#]=!=List) &;
*)

SpeciesQ := UpperCaseQ[StringTake[ToString[#],1]]&;

ModelsQ := MemberQ[Models,#]&;

ExternalQ := (Head[#]===Rule&&First[#]===ExternalSpecies&&Head[Last[#]]===List)&;


SymbolQ := Head[#]===Symbol&;

PlusQ := Head[#]===Plus&;

TimesQ := Head[#]===Times&;

SubscriptQ := Head[#]===Subscript&;

SuperscriptQ := Head[#]===Superscript&;

RightArrowQ := (Head[#]===RightArrow&&Length[#]===2&&
				And@@Map[StringQ[#]||SymbolQ[#]||PlusQ[#]||TimesQ[#]||SubscriptQ[#]||SuperscriptQ[#]&,#/.RightArrow->List])&;

LeftArrowQ := (Head[#]===LeftArrow&&Length[#]===2&&
				And@@Map[StringQ[#]||SymbolQ[#]||PlusQ[#]||TimesQ[#]||SubscriptQ[#]||SuperscriptQ[#]&,#/.LeftArrow->List])&;

EquilibriumQ := (Head[#]===Equilibrium&&Length[#]===2&&
				And@@Map[StringQ[#]||SymbolQ[#]||PlusQ[#]||TimesQ[#]||SubscriptQ[#]||SuperscriptQ[#]&,#/.Equilibrium->List])&;

ReverseEquilibriumQ := (Head[#]===ReverseEquilibrium && Length[#]===2 && 
						And@@Map[StringQ[#]||SymbolQ[#]||PlusQ[#]||TimesQ[#]||SubscriptQ[#]||SuperscriptQ[#]&,#/.ReverseEquilibrium->List])&;

ReactionsToList[{reactions__?RightArrowQ}|Sequence[reactions__?RightArrowQ]]:=
		DeleteDuplicates[{reactions}]/.RightArrow->List;

ReactionsToList[{reactions__?LeftArrowQ}|Sequence[reactions__?LeftArrowQ]] := 
		Reverse/@(DeleteDuplicates[{reactions}]/.LeftArrow->List);

ReactionsToList[{reactions__?EquilibriumQ}|Sequence[reactions__?EquilibriumQ]]:=
		Sequence@@({#,Reverse[#]}/.Equilibrium->List)&/@DeleteDuplicates[{reactions}];
		(*Flatten[{{First[#],Last[#]},{Last[#],First[#]}}&/@DeleteDuplicates[{reactions}],1];*)
ReactionsToList[{reactions__?ReverseEquilibriumQ}|Sequence[reactions__?ReverseEquilibriumQ]]:=
		Sequence@@({Reverse[#],#}/.ReverseEquilibrium->List)&/@DeleteDuplicates[{reactions}];

ReactionsToList[{reactions__}|Sequence[reactions__]]:=
	Module[{form, sp, e,re, a},
			form = DeleteDuplicates[ToCanonicalForm[reactions]];
			DeleteDuplicates[
				Flatten[Map[
					Switch[#,
						_?RightArrowQ,#,
						_?LeftArrowQ,Reverse[#]/.LeftArrow->RightArrow,
						_?EquilibriumQ,{(e=#/.Equilibrium->RightArrow),Reverse[e]},
						_?ReverseEquilibrium,{(re=Reverse[#/.ReverseEquilibrium->RightArrow]),Reverse[re]},
						_,
						(sp = Flatten[ToBoxes[#,StandardForm]/.RowBox->List/.Thread[{"(",")","{","}",","}:>Sequence[]]];(*MakeBoxes*)
						a = Sort[Flatten[Position[sp,#]&/@{"\[Equilibrium]","\[RightArrow]","\[ReverseEquilibrium]","\[LeftArrow]"}]];
						(Partition[Riffle[ReleaseHold[MakeExpression[RowBox[Take[sp,#+{1,-1}]],StandardForm]]&/@Partition[Join[{0},a,{Length[sp]+1}],2,1],Part[sp,a]],3,2]
						/.{{x_,"\[Equilibrium]",y_}->Sequence[RightArrow[x,y],RightArrow[y,x]],{x_,"\[ReverseEquilibrium]",y_}->Sequence[RightArrow[y,x],RightArrow[x,y]],{x_,"\[LeftArrow]",y_}->RightArrow[y,x],{x_,"\[RightArrow]",y_}->RightArrow[x,y]}))]&,form]
						/.RightArrow[A_,A_]->Sequence[]]]/.RightArrow->List
];

(*
ReactionsToList[List[Reactions__]|Sequence[Reactions__]] := 
	Module[{ $RecursionLimit = Infinity, conversion },
		conversion[RightArrow[A__,B__]] := conversion/@{A,"->",B};
		conversion[LeftArrow[A__]] := conversion[Fold[LeftArrow,First[{A}],Rest[{A}]]];
		conversion[LeftArrow[A_,B_]] := conversion/@{A,"<-",B};
		conversion[Equilibrium[A_,B__]] := conversion/@{A,"\[Equilibrium]",If[Length[{B}]>1,Riffle[{B},"\[Equilibrium]"],conversion[B]]};
		conversion[ReverseEquilibrium[A_,B__]] := conversion/@{A,"\[ReverseEquilibrium]",If[Length[{B}]>1,Riffle[{B},"\[ReverseEquilibrium]"],conversion[B]]};
		conversion[Sequence[A__]] := A;
		DeleteDuplicates[
			Flatten[(Partition[#,3,2]&/@ Map[Flatten[conversion[#]]&,ToCanonicalForm[Reactions]]) 
					/. {{A_,"->",B_}->{{A,B}},{A_,"<-",B_}->{{B,A}},{A_,"\[Equilibrium]",B_}->{{A,B},{B,A}},{A_,"\[ReverseEquilibrium]",B_}->{{B,A},{A,B}}},2]
						] /. {A_?NotListQ, A_?NotListQ}->Sequence[]
	];
*)


Rdata[{reactions___},externals_] := Rdata[{reactions},externals] = 
	Module[
			{ exs, builtin, m, s, c, c0, rs, fhj, fhjc, sp,
			  spp, e, ab, rd, nloc, lloc, sloc, coefflist, vg },
			exs = DeleteDuplicates[Prepend[If[VectorQ[externals],
											externals,
											ExternalSpecies/.externals
										],"0"]];
			If[(m=Cases[{reactions},_String])=!={},
				builtin = Check[(#->GetProblem[#])&/@ m, Abort[];, GetProblem::"nvmod"];(*ReactionsData message?*)
				Rdata[DeleteDuplicates[Join[Flatten[{reactions}/.builtin],{reactions}/.Thread[m->Sequence[]]]],externals],
				s = ReactionsToList[reactions] /.(0->"0"); (*"0" - zero complex*)
				coefflist[complex_] := Flatten[{complex/.Thread[exs->0]}/.Plus->List] /. r_*A_?StringQ -> {A,r} /. A_?StringQ*r_ -> {A,r} /. r_*A_?SpeciesQ -> {A,r} /. A_?SpeciesQ*r_ -> {A,r};
				Dispatch[{
					"species" -> (sp = 
									(spp = Union[Flatten[(s /. r_*A_?StringQ -> A /. A_?StringQ*r_ -> A) /. r_*A_?SpeciesQ -> A /. A_?SpeciesQ*r_ -> A /.Plus->List]])
									/.Thread[exs->Sequence[]]), 
									(*(sp = Complement[spp = contspecs[s],exs])*)
					"M" -> Length[sp],
					"externalspecies" -> (e = If[MemberQ[spp,"0"], exs, Rest[exs]]), (*Union*)
					"E" -> Length[e],
					"reactionsteps" -> (rs = (#/.List->RightArrow)&/@s),
					"R" -> Length[rs],
					"complexes" -> 
								(c0 = (Union[(c=Union@@s)/.Thread[exs->0]]/.(0->"0"))), (* ,c}  DeleteDuplicates*)
					"fhjgraph" -> 
								(fhj = FromOrderedPairs[rs /. RightArrow[c1_,c2_] :> {Position[c, c1][[1, 1]], Position[c, c2][[1, 1]]}]),
								(*ShowLabeledGraph[fhj, c, PlotRange -> All]*)
					"fhjconnectedcomponents" -> 
						{
						(fhjc = WeaklyConnectedComponents[fhj] /. i_Integer :> c[[i]]),(*reactionsteps*)
						ConnectedComponents[MakeUndirected[fhj]] /. i_Integer :> c[[i]]
						},
					"fhjstronglyconnectedcomponents" -> (StronglyConnectedComponents[fhj] /. i_Integer :> c[[i]]),
					"n" -> (nloc=Length[c0]),
					"l" -> (lloc=Length[fhjc]),
					"s" -> (sloc=MatrixRank[First[Differences[ab=Transpose/@ Transpose[Outer[Coefficient,s,sp]]]]]),
					"deficiency" -> "\[Delta]=n-l-s="<>ToString[nloc]<>"-"<>ToString[lloc]<>"-"<>ToString[sloc]<>"="<>ToString[nloc-lloc-sloc],
					"\[Alpha]" -> Part[ab,1],
					"\[Beta]" -> Part[ab,2],
					"\[Gamma]" -> First[Differences[ab]],
					"reactionorder" -> Max[Total /@ Transpose[Part[ab,1]]],
					"variables" -> (Subscript[Global`c, #] & /@ sp),
					"volpertgraph" -> 
						Flatten[(Thread[#->(coefflist[Last[#]]/.(0->{}))]~Join~Thread[(coefflist[First[#]]/.(0->{}))->#] &/@ rs) 
						/. {Rule[{},A_]:>Sequence[], Rule[A_,{}]:>Sequence[]} /. Rule[A_RightArrow,B_List]:>{A->First[B],Last[B]} /. Rule[A_List,B_RightArrow]:>{First[A]->B,Last[A]},1]
				}]
			]
	];

(*
{FromOrderedPairs[vg /. Rule[c1_,c2_] :>If[Head[c1]===RightArrow,{Length[sp]+Position[rs, c1][[1, 1]], Position[sp, c2][[1, 1]]},
																{Position[sp, c1][[1, 1]], Length[sp]+Position[rs, c2][[1, 1]]}]],Join[sp,rs]}
*)


(* ::Subsubsection:: *)
(*ToCanonicalForm, ToReversible, ReactionsData*)


ToCanonicalForm[{reactions__}|Sequence[reactions__]]:=
	Flatten[{reactions} /. Reactions] /. 
					  Join[Thread[{LeftRightArrow,DoubleLeftRightArrow,LongLeftRightArrow,DoubleLongLeftRightArrow,RightArrowLeftArrow}->Equilibrium],
									{LeftArrowRightArrow->ReverseEquilibrium},
									Thread[{Rule,ShortRightArrow,DoubleRightArrow,LongRightArrow,DoubleLongRightArrow}->RightArrow],
									Thread[{ShortLeftArrow,DoubleLeftArrow,LongLeftArrow,DoubleLongLeftArrow}->LeftArrow]];

(*GetProblem, Models, azokat is kezelje*)

ToReversible[{reactions__}|Sequence[reactions__]] := ToCanonicalForm[reactions]/.RightArrow->Equilibrium/.LeftArrow->Equilibrium;


PropertiesReactionsData = 
	{
		"species","M","externalspecies","E","reactionsteps","R","complexes",
		"fhjgraph","fhjconnectedcomponents","fhjstronglyconnectedcomponents",
		"n","l","s","deficiency","\[Alpha]","\[Beta]","\[Gamma]","reactionorder","variables","volpertgraph"
	};


ReactionsData::badarg = "Illegal argument of function ReactionsData.";
ReactionsData::badname = "Argument `1` contains non-identified properties. See the possibilities in \
\!\(\*\nStyleBox[\"PropertiesReactionsData\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\).";
ReactionsData::wrreac = "Argument `1` may not have a correct form. The list of reactions must contain the operations Plus, Times, String, Symbol and many kinds of arrows only\ 
in a reasonable form. For help check the palette \!\(\*\nStyleBox[\"OpenReactionKineticsPalette\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\).";

ReactionsData[modelname_?ModelsQ][] := ReactionsData[{modelname},{}][];
ReactionsData[modelname_?ModelsQ,(externals_?VectorQ | externals_?ExternalQ)][] := ReactionsData[{modelname},externals][];
ReactionsData[{reactions__}][] := Normal[Rdata[{reactions},{}]];
ReactionsData[{reactions__},(externals_?VectorQ | externals_?ExternalQ)][] := Normal[Rdata[{reactions},externals]];

ReactionsData[modelname_?ModelsQ]["Properties"] := PropertiesReactionsData;
ReactionsData[modelname_?ModelsQ,(externals_?VectorQ | externals_?ExternalQ)]["Properties"] := PropertiesReactionsData;
ReactionsData[{reactions__}]["Properties"] := PropertiesReactionsData;
ReactionsData[{reactions__},(externals_?VectorQ | externals_?ExternalQ)]["Properties"] := PropertiesReactionsData;

ReactionsData[modelname_?ModelsQ][data__?StringQ] := ReactionsData[{modelname},{}][data];
ReactionsData[modelname_?ModelsQ,(externals_?VectorQ | externals_?ExternalQ)][data__?StringQ] := ReactionsData[{modelname},externals][data];
ReactionsData[modelname_?ModelsQ][{data__?StringQ}] := ReactionsData[{modelname},{}][data];
ReactionsData[modelname_?ModelsQ,(externals_?VectorQ | externals_?ExternalQ)][{data__?StringQ}] := ReactionsData[{modelname},externals][data];

ReactionsData[{reactions__}][data__?StringQ] := ReactionsData[{reactions},{}][data];
ReactionsData[{reactions__}][{data__?StringQ}] := ReactionsData[{reactions},{}][data];
ReactionsData[{reactions__},(externals_?VectorQ | externals_?ExternalQ)][data__?StringQ] := 
	If[And@@(MemberQ[PropertiesReactionsData,#]&/@{data}),
			Catch[Part[ReplaceAll[{data},Check[Rdata[{reactions},externals],Throw[Message[ReactionsData::"wrreac",{reactions}]; Abort[];]]],
				If[Length[{data}]>1,1;;Length[{data}],1]]],
			Message[ReactionsData::"badname",{data}];
			$Failed
	];
ReactionsData[{reactions__},(externals_?VectorQ | externals_?ExternalQ)][{data__?StringQ}] := ReactionsData[{reactions},externals][data];

ReactionsData[{reactions__},data__?StringQ] := ReactionsData[{reactions},{}][data];
ReactionsData[{reactions__},(externals_?VectorQ | externals_?ExternalQ),data__?StringQ] := ReactionsData[{reactions},externals][data];

Format[ReactionsData[modelname_?ModelsQ]] := 
	"The \!\(\*\nStyleBox[\"ReactionsData\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\) object is generated for the model "<>
	modelname <>" with no external species, beside 0 and \"0\".\n"<>
	"For more help check the function \!\(\*\nStyleBox[\"ReactionsData\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\), "<>
	"the palette of names: \!\(\*\nStyleBox[\"OpenReactionKineticsNamesPalette\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\) "<>
	"or the list \!\(\*\nStyleBox[\"names\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\)."

Format[ReactionsData[modelname_?ModelsQ,(externals_?VectorQ | externals_?ExternalQ)]] := 
	"The \!\(\*\nStyleBox[\"ReactionsData\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\) object is generated for the reaction(s)/model(s) "<>
	modelname <>" with external species " <> ToString[If[VectorQ[externals],externals,ExternalSpecies/.externals],StandardForm]<>".\n"<>
	"For more help check the function \!\(\*\nStyleBox[\"ReactionsData\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\), "<>
	"the palette of names: \!\(\*\nStyleBox[\"OpenReactionKineticsNamesPalette\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\) "<>
	"or the list \!\(\*\nStyleBox[\"names\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\).";

Format[ReactionsData[{reactions__}]] := 
	"The \!\(\*\nStyleBox[\"ReactionsData\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\) object is generated for the reaction(s)/model(s) "<>
	If[Length[{reactions}]===1,ToString[{reactions},StandardForm],"{"<>ToString[First[{reactions}],StandardForm]<>",...,"<>ToString[Last[{reactions}],StandardForm]<>"}"]<>
	" with no external species, beside 0 and \"0\".\n"<>
	"For more help check the function \!\(\*\nStyleBox[\"ReactionsData\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\), "<>
	"the palette of names: \!\(\*\nStyleBox[\"OpenReactionKineticsNamesPalette\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\) "<>
	"or the list \!\(\*\nStyleBox[\"names\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\)."

Format[ReactionsData[{reactions__},(externals_?VectorQ | externals_?ExternalQ)]] := 
	"The \!\(\*\nStyleBox[\"ReactionsData\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\) object is generated for the reaction(s)/model(s) "<>
	If[Length[{reactions}]===1,ToString[{reactions},StandardForm],"{"<>ToString[First[{reactions}],StandardForm]<>",...,"<>ToString[Last[{reactions}],StandardForm]<>"}"]<>" with external species "<>
	ToString[If[VectorQ[externals],externals,ExternalSpecies/.externals],StandardForm]<>".\n"<>
	"For more help check the function \!\(\*\nStyleBox[\"ReactionsData\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\), "<>
	"the palette of names: \!\(\*\nStyleBox[\"OpenReactionKineticsNamesPalette\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\) "<>
	"or the list \!\(\*\nStyleBox[\"names\",\nFontFamily->\"Courier\",\nFontColor->RGBColor[0, 0, 1]]\).";

ReactionsData[___][___] := (Message[ReactionsData::"badarg"]; $Failed)


(* ::Subsubsection::Closed:: *)
(*WeaklyReversibleQ*)


WeaklyReversibleQ[{reactions__}|Sequence[reactions__]] := 
		SymmetricQ[TransitiveClosure[Check[ReactionsData[{reactions},"fhjgraph"],$Failed,{ReactionsData::wrreac,ReactionsData::badarg}]]];


(* ::Subsubsection::Closed:: *)
(*ShowFHJGraph, ShowVolpertGraph*)


GraphPlotFunctionQ:=MemberQ[{GraphPlot,LayeredGraphPlot,GraphPlot3D,TreePlot},#]&;

ShowFHJGraph::badarg = "Illegal argument of function ShowFHJGraph.";
ShowFHJGraph::ccols = "The given colours may have not in the correct vector form or have wrong size.";
ShowFHJGraph::srates = "The given list of rate constants may have wrong size.";
ShowFHJGraph::sccomps = "The given list of colours for the strongly connected components may have wrong form or length or it is also given complex colours.";

Options[ShowFHJGraph]:={ComplexColours->{},StronglyConnectedComponentsColours->{}};

ShowFHJGraph[{reactions__},rates_List:{},opts___?OptionQ]:=ShowFHJGraph[GraphPlot,{reactions},rates,opts];

ShowFHJGraph[graphfunc_?GraphPlotFunctionQ,{reactions__},rates_List:{},opts___?OptionQ] := 
	Module[{ cc, n, r, rsteps, cxs, fhjscc, nopt, c, sc, rscc, rs },
			cc = FilterRules[{opts},Options[ShowFHJGraph]];
			{n, r, rsteps, cxs, fhjscc} = 
					Check[ReactionsData[{reactions},"n","R","reactionsteps","complexes","fhjstronglyconnectedcomponents"] /. RightArrow->Rule,Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
			nopt = Sequence @@ FilterRules[{opts},Options[graphfunc]];
			c = ComplexColours /. cc;
			sc = StronglyConnectedComponentsColours /. cc;
			If[sc=!={} && sc =!= StronglyConnectedComponentsColours,
				If[(VectorQ[sc]&&Length[fhjscc] === Length[sc])&&(c === {} || c === ComplexColours),
					rscc = rsteps /. Flatten[Thread[First[#]->Thread[Style[First[#],Italic,Last[#]]]] &/@ Thread[{fhjscc,sc}]];
						If[rates==={},
							graphfunc[rscc,nopt],
							If[r===Length[rates],
								graphfunc[Thread[{rscc,rates}],nopt],
								Message[ShowFHJGraph::"srates"];
								$Failed
							]
						],
					Message[ShowFHJGraph::"sccomps"];
					$Failed
				],
				If[c=!={} && c =!= ComplexColours,
					If[VectorQ[c] && n===Length[c],
						rs = rsteps /. (First[#]->Style[First[#],Italic,Last[#]] &/@ Thread[{cxs,c}]);
						If[rates==={},
							graphfunc[rs,nopt],
							If[r===Length[rates],
								graphfunc[Thread[{rs,rates}],nopt],
								Message[ShowFHJGraph::"srates"];
								$Failed
							]
						],
						Message[ShowFHJGraph::"ccols"];
						$Failed
					],
					If[rates==={},
						GraphPlot[rsteps,nopt],
						If[r===Length[rates],
								graphfunc[Thread[{rsteps,rates}],nopt],
								Message[ShowFHJGraph::"srates"];
								$Failed
							]
					]
				]
			]
	];

ShowFHJGraph[___] := (Message[ShowFHJGraph::"badarg"]; $Failed)


Options[ShowVolpertGraph]:={ExternalSpecies->{}}

ShowVolpertGraph::badarg = "Illegal argument of function ShowVolpertGraph.";
ShowVolpertGraph::funcarg = "Argument `1` is not a valid graph plot function. Try GraphPlot, LayeredGraphPlot, GraphPlot3D, TreePlot, ShowGraph or ShowLabeledGraph.";

ShowVolpertGraph[{reactions__},opts___?OptionQ] := ShowVolpertGraph[GraphPlot,{reactions},opts];

ShowVolpertGraph[graphfunc_Symbol,{reactions__},opts___?OptionQ] := 
	Module[{vg, exs},
		exs = Sequence@@FilterRules[{opts},Options[ShowVolpertGraph]];
		If[MemberQ[{GraphPlot,LayeredGraphPlot,GraphPlot3D,TreePlot,ShowGraph,ShowLabeledGraph},graphfunc],
			vg = Check[ReactionsData[{reactions},exs,"volpertgraph"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
			Switch[graphfunc,
					ShowGraph,ShowGraph[ToCombinatoricaGraph[vg],Sequence@@FilterRules[{opts},Options[ShowGraph]]],
					ShowLabeledGraph,ShowLabeledGraph[ToCombinatoricaGraph[vg],Sequence@@FilterRules[{opts},Options[ShowLabeledGraph]]],
					_,graphfunc[vg,Sequence@@FilterRules[{opts},Options[graphfunc]]]
			],
			Message[ShowVolpertGraph::"funcarg",graphfunc];
			$Failed
		]
	];

ShowVolpertGraph[___] := (Message[ShowVolpertGraph::"badarg"]; $Failed)


(* ::Subsubsection::Closed:: *)
(*Atoms, ToAtomMatrix, FromAtomMatrix and own funcions*)


Atoms = 
	{
	  "Ubn","Ubu","Uub","Uue","Uuh","Uuo","Uup","Uuq","Uus","Uut",
	  "Ac","Ag","Al","Am","Ar","As","At","Au","Ba","Be","Bh","Bi","Bk","Br","Ca","Cd","Ce","Cf","Cl","Cm","Co","Cr","Cs","Cu",
	  "Db","Ds","Dy","Er","Es","Eu","Fe","Fm","Fr","Ga","Gd","Ge","He","Hf","Hg","Ho","Hs","In","Ir","Kr","La","Li","Lr","Lu",
	  "Md","Mg","Mn","Mo","Mt","Na","Nb","Nd","Ne","Ni","No","Np","Os","Pa","Pb","Pd","Pm","Po","Pr","Pt","Pu","Ra","Rb","Re","Rf","Rg","Rh","Rn","Ru",
	  "Sb","Sc","Se","Sg","Si","Sm","Sn","Sr","Ta","Tb","Tc","Te","Th","Ti","Tl","Tm","Xe","Yb","Zn","Zr",
	  "B","C","F","H","I","K","N","O","P","S","U","V","W","Y"
	};
(* sorrend! *)


NestedMoleculeQ[s_?StringQ] := StringFreeQ[NestWhile[StringReplace[#,("("~~x___~~")"/;StringCount[x,{"(",")"}]===0)->""]&,s,UnsameQ[##]&,2],{"(",")"}];

MoleculeToAtoms[s_?StringQ,ats_?VectorQ]:=
	StringCases[s, 
			Join[" "~~r1___?DigitQ~~#:>ToExpression[#<>If[r1=="","1",r1]]*"charge"&/@{"+","-"},
			#~~r2___?DigitQ:>ToExpression[If[r2 == "","1",r2]]*#&/@ats,
			{"("~~x___?NestedMoleculeQ~~")"~~r3__?DigitQ:>Sequence@@Times[ToExpression[If[r3=="","1",r3]],MoleculeToAtoms[x,ats]]}],
				Overlaps->False];

(*tobbszorosen beagyazott molekulakra*)


ToAtomMatrix[molecules_?VectorQ,OptionsPattern[{Data->{},FormattedOutput->False}]]:=
	Module[{ ats, l, toa, at, d = OptionValue[Data] },
		ats := Append[Reverse[Sort[Union@@StringCases[molecules,Atoms],StringLength[#1]<StringLength[#2]&]],"charge"];
		l := Length[ats];
		toa := {Map[Total[MoleculeToAtoms[#,ats]]&,molecules]/.Thread[ats->(UnitVector[l,#]&/@Range[l])],ats};
		If[OptionValue[FormattedOutput],
			at = {"atoms" -> Column[Last[toa]],
				  "atommatrix" -> Grid[MapThread[Prepend,{Prepend@@toa,Prepend[molecules,"Molecules\[Backslash]Atoms"]},1],Dividers->{{False,True},{False,True}}]};
			Column[If[d==={},
				{"atoms","atommatrix"}/.at,
				d/.at
			]],
			If[d==={},
				Thread[{Global`atommatrix, Global`atoms}->toa],
				(Symbol["Global`"<> #]/.ToAtomMatrix[molecules]) &/@ d
			]
		]
	];

AtomConservingQ[{reactions__},externals_List:{}] :=
	Module[{ls, alpha, beta, atommatrix},
		{ls, alpha, beta} = Check[ReactionsData[{reactions},externals,"species","\[Alpha]","\[Beta]"],Abort[];,{ReactionsData::wrreac,ReactionsData::badarg}];
		atommatrix = First[ToAtomMatrix[ls,Data->{"atommatrix"}]];
		(*Transpose[a].atommatrix == Transpose[b].atommatrix*)
		(#===ConstantArray[0,Length[First[atommatrix]]]) &/@ (Transpose[alpha-beta].atommatrix)
	];


AtomsQ[v_?VectorQ] := And@@(MemberQ[Atoms,#]&/@v);

FromAtomMatrix[atommatrix_?MatrixQ,atoms_?AtomsQ] := 
	Inner[
		Switch[{#1,#2},{0,_},"",{_,"charge"},StringJoin[" ",If[Abs[#1]===1,"",Abs[#1]],If[#1<0,"-","+"]],{1,_},#2,_,StringJoin[#2,ToString[#1]]]&,
		atommatrix,
		Append[atoms,"charge"],
		StringJoin
	];


End[ ];


(* ::Subsubsection::Closed:: *)
(*protect functions*)


Protect[
	Global`atommatrix,(**)
	Global`atoms,(**)

	Global`c,
	Global`r
];


SetAttributes[
	{
		ComplexColours,
		Data,(**)
		ExternalSpecies,
		FormattedOutput,
		StronglyConnectedComponentsColours,
	
		ToCanonicalForm,
		ToReversible,
		PropertiesReactionsData,
		ReactionsData,
		WeaklyReversibleQ,
	
		ShowFHJGraph,
		ShowVolpertGraph,
	
		Atoms,
		ToAtomMatrix,
		AtomConservingQ,
		AtomsQ,
		FromAtomMatrix
	},
	{Protected,ReadProtected}
];


(* ::Subsubsection:: *)
(*end*)


EndPackage[ ];
